import React, { Component } from "react";
import { HashRouter as Router, Route, Switch } from "react-router-dom";
import "bootstrap/dist/css/bootstrap.min.css";
import "./static/styles/style.scss"
import LandingPage from "./pages/LandingPage";
import CreateAccount from "./pages/accounts/CreateAccount";
import Login from "./pages/accounts/Login";
import ForgotPassword from "./pages/accounts/ForgotPassword";
import ResetPassword from "./pages/accounts/ResetPassword";
import DashBoard from "./pages/DashBoard";
import CreateEvent from "./pages/chair/CreateEvent";
import EventDetail from "./pages/EventDetail";
import CreateRubric from "./pages/chair/CreateRubric";
import AssignGroups from "./pages/chair/AssignGroups";
import ScoreReport from "./pages/chair/ScoreReport";
import AssignedEntries from "./pages/judges/AssignedEntries";
import GradeEntry from "./pages/judges/GradeEntry";
import EventRegister from './pages/contestants/EventRegister';
import {UserContextProvider} from './components/Context';
import { createMuiTheme, ThemeProvider } from '@material-ui/core/styles';
import { ErrorScreen } from './components/Screens';

const theme = createMuiTheme({
  palette: {
    primary: {
      main: "#5B4EC3"
    }
  }
});

class Evaluate extends Component {
  render() {
    return (
      <UserContextProvider>
        <ThemeProvider theme={theme}>
          <Router>
            <Switch>
              <Route exact path="/" component={LandingPage}></Route>
              <Route exact path="/createaccount" component={CreateAccount}></Route>
              <Route exact path="/editaccount" component={CreateAccount}></Route>
              <Route exact path="/login" component={Login}></Route>
              <Route exact path="/forgotpassword" component={ForgotPassword}></Route>
              <Route exact path="/resetpassword/:code" component={ResetPassword}></Route>
              <Route exact path="/dashboard" component={DashBoard}></Route>

              <Route exact path="/createevent" component={CreateEvent}></Route>
              <Route exact path="/editevent/:eventId" component={CreateEvent}></Route>

              <Route exact path="/event/:eventId" component={EventDetail}></Route>
              <Route exact path="/event/:eventId/createrubric" component={CreateRubric}></Route>
              <Route exact path="/event/:eventId/editrubric" component={CreateRubric}></Route>
              <Route exact path="/event/:eventId/assigngroups" component={AssignGroups}></Route>
              <Route exact path="/event/:eventId/scorereport" component={ScoreReport}></Route>
              <Route exact path="/event/:eventId/chair/:judgeId/entry/:entryId/gradeentry" component={GradeEntry}></Route>

              <Route exact path="/event/:eventId/judge/:judgeId/assignedentries" component={AssignedEntries}></Route>
              <Route exact path="/event/:eventId/judge/:judgeId/entry/:entryId/gradeentry" component={GradeEntry}></Route>
              
              <Route exact path="/event/:eventId/eventregister" component={EventRegister}></Route>
              <Route exact path="/event/:eventId/entry/:entryId" component={EventRegister}></Route>

              <Route exact path="/*" component={ErrorScreen}></Route>
            </Switch>
          </Router>
        </ThemeProvider>
      </UserContextProvider>
    );
  }
}

export default Evaluate;
