import React from "react";
import { Link } from "react-router-dom";
import { Button, IconButton } from "@material-ui/core";
import AddIcon from '@material-ui/icons/Add'
import DeleteIcon from '@material-ui/icons/Delete'
import EditIcon from '@material-ui/icons/Edit';
import SaveIcon from "@material-ui/icons/Save";
import ExpandMoreIcon from '@material-ui/icons/ExpandMore';
import ExpandLessIcon from '@material-ui/icons/ExpandLess';


const button = {
  outline: 'none',
  marginRight: '10px'
}

const iconButton = {
  outline: 'none',
  margin: '0px'
}

function AddButton(props) {
  return(
  <IconButton style={iconButton} size="small" color="primary" onClick={props.onClick}>
    <AddIcon />
  </IconButton>
  );
}

function DeleteButton(props) {
  return (
    <IconButton style={iconButton} size="small" color="secondary" onClick={props.onClick}>
      <DeleteIcon />
    </IconButton>
  );
}

function EditButton(props) {
  return (
    <Link to={props.link}>
      <IconButton style={iconButton} size="small" color="secondary" onClick={props.onClick}>
        <EditIcon />
      </IconButton>
    </Link>
  );
}

function ExpandButton(props) {
  let icon = (<ExpandMoreIcon></ExpandMoreIcon>);
  if (props.isShowing) {
    icon = (<ExpandLessIcon></ExpandLessIcon>);
  }

  return (
    <IconButton style={iconButton} size="small" color="primary" onClick={props.onClick}>
      {icon}
    </IconButton>
  );
}

function PrimaryButton(props) {
  return (
    <Button
      style={button}
      variant="contained"
      color="primary"
      type={props.type}
      onClick={props.onClick}
      startIcon={props.startIcon}
      fullWidth={props.fullWidth}
      disabled={props.disabled}
    >
      {props.children}
    </Button>
  );
}

function SaveButton(props) {
  return (
    <Button
      style={button}
      variant="contained"
      color="primary"
      type={props.type}
      onClick={props.onClick}
      startIcon={<SaveIcon />}
      fullWidth={props.fullWidth}
      aria-label="save button"
      disabled={props.disabled}
    >
      Save
    </Button>
  );
}

function SecondaryButton(props) {
  return (
    <Button
      style={button}
      variant="contained"
      color="secondary"
      type={props.type}
      onClick={props.onClick}
      startIcon={props.startIcon}
      fullWidth={props.fullWidth}
      disabled={props.disabled}
    >
      {props.children}
    </Button>
  );
}

//only used in layout:
function LoginButton(props){
  const login = {
    color: 'white',
    fontSize: '16px',
    fontWeight: 'bold',
  }
  return(
    <Button
    onClick={props.onClick}
    style={{...login,...button}}>
      Login
    </Button>
  )
}

//only used in layout:
function SignUpButton(props){
  const signup = {
    color: '#000',
    fontSize: '16px',
    fontWeight: 'bold',
  }

  return(
    <Button
    onClick={props.onClick}
    style={{...signup,...button}}
    >
      Sign Up
    </Button>
  )
}

export {AddButton, DeleteButton, EditButton, ExpandButton, PrimaryButton, SaveButton, SecondaryButton, LoginButton, SignUpButton}
