import React, {Component} from 'react';
import {get} from '../Utils';

const UserContext = React.createContext();

class UserContextProvider extends Component {

  state = {
    name: null
  };

  setName(name) {
    this.setState({name: name});
  }

  async getName() {
    let {status, body} = await get("/login/context");
    if (status === 200) {
      this.setState({name: body.fullName});
      return true;
    }
    return false;
  }

  async logout() {
    let response = await fetch("/login/logout");
    if (response.status === 200) {
      this.setState({name: null});
      return true;
    }
    return false;
  }

  render() {
    return (
      <UserContext.Provider
        value={{
          name: this.state.name,
          setName: (name) => this.setName(name),
          getName: () => this.getName(),
          logout: () => this.logout()
        }}
      >
        {this.props.children}
      </UserContext.Provider>
    );
  }

}

class UserContextConsumer extends Component {

  render() {
    return (
      <UserContext.Consumer>
        {this.props.children}
      </UserContext.Consumer>
    );
  }

}

export {UserContext, UserContextConsumer, UserContextProvider};
