import React from 'react';
import {PrimaryButton} from './Buttons'
import * as FileSaver from 'file-saver';
import * as XLSX from 'xlsx';

export const ExportXLSX = ({csvData, fileName, categoryList}) => {
    const fileType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=UTF-8';
    const fileExtension = '.xlsx';


    const exportToXLSX = (csvData, fileName, categoryList) => {
        const wb = { Sheets: {}, SheetNames: []}
        for(let group in csvData){
            var categoryLength = categoryList.length;
            var heading = []
            var head1 = ["", "", ""]
            var dataArray = []
            var judgeList = csvData[group][0].scoredRubrics
            var judgeNum = 0;
            var judges = []
            var wscols = [
                {wch:70},
                {wch:15},
                {wch:15}
            ];
            
            for(let j in judgeList){
                let judge = judgeList[j].judge.name;
                //add head1
                head1.push(judge)
                for(let i = 0; i < categoryLength - 1; i++){
                    head1.push("")
                }
                judges.push(j)
                judgeNum++;
            }
            heading.push(head1);
            var head2 = ["Entry Title", "Total Score", "Average Score"]
            var i = 0;
            while(i < categoryLength*judgeNum){
                for(let cat of categoryList){
                    // add head2
                    var wch = {wch: 15}
                    wscols.push(wch)
                    head2.push(cat.name)
                }
                i += categoryLength;
            }
            heading.push(head2)
            var entries = csvData[group]
            for(let e in entries){
                var entryData = []
                var entry = entries[e]
                entryData.push(entry.title)
                entryData.push(entry.total)
                entryData.push(parseInt(entry.average))
                for(let j of judges){
                    entryData.push(...entry.scoredRubrics[j].scores)
                }
                dataArray.push(entryData)
            }
            const gn = parseInt(group) + 1;
            var sheetName = "Group "+gn; //Sheet Name
            wb.SheetNames.push(sheetName);
            var ws = XLSX.utils.aoa_to_sheet(heading)
            XLSX.utils.sheet_add_aoa(ws, dataArray, {
                origin: "A3",
                margins: "wide"
            })
            var j = 3;
            while(j < judgeNum*categoryLength + 3){
                var merge2 = { s: {r: 0, c: j}, e: {r: 0, c: categoryLength + j - 1}}
                if(!ws['!merges']) ws['!merges'] = [];
                ws['!merges'].push(merge2);
                j += categoryLength; 
            }
            ws['!cols'] = wscols;
            wb.Sheets[sheetName] = ws;
        }
        const excelBuffer = XLSX.write(wb, {bookType: 'xlsx', type: 'array'})
        const data = new Blob([excelBuffer], {type: fileType})
        FileSaver.saveAs(data, fileName + fileExtension)
    }

    return (
        <PrimaryButton onClick={(e) => exportToXLSX(csvData, fileName, categoryList)}>Export to XLSX</PrimaryButton>
    )
}