import React, { useContext } from "react";
import { Link } from "react-router-dom";
import { makeStyles, withStyles } from "@material-ui/core/styles";
import AppBar from "@material-ui/core/AppBar";
import Toolbar from "@material-ui/core/Toolbar";
import ArrowDropDownIcon from "@material-ui/icons/ArrowDropDown";
import {
  Menu,
  MenuItem,
  ListItemText
} from "@material-ui/core";
import { LoginButton, SignUpButton, PrimaryButton } from "./Buttons";
import { UserContext } from "../components/Context";
import Logo from "../static/images/inline-ww.png";

const useStyles = makeStyles((theme) => ({
  root: {
    margin: 0,
    flexGrow: 1,
  },
  navbar: {
    background: "#5B4EC3",
    boxShadow: "none"
  },
  logo: {
    width: "40%",
    height: "auto",
    [theme.breakpoints.up('sm')]: {
      width: "30%"
    },
    [theme.breakpoints.up('md')]: {
      width: "20%"
    },
    [theme.breakpoints.up('lg')]: {
      width: "15%"
    },
    [theme.breakpoints.up('xl')]: {
      width: "10%"
    }
  },
  register: {
    position: "absolute",
    right: "1rem"
  },
  footer: {
    marginTop: '1rem',
    background: "transparent",
    width: "100%",
    textAlign: "center"
  }
}));

/*******************Menu Item*******************************/
const StyledMenu = withStyles({
  paper: {
    border: "1px solid #d3d4d5"
  }
})(props => (
  <Menu
    elevation={0}
    getContentAnchorEl={null}
    anchorOrigin={{
      vertical: "bottom",
      horizontal: "center"
    }}
    transformOrigin={{
      vertical: "top",
      horizontal: "center"
    }}
    {...props}
  />
));

const StyledMenuItem = withStyles(theme => ({
  root: {
    "& .MuiListItemIcon-root, & .MuiListItemText-primary": {
      color: theme.palette.primary.main
    },
    "&:focus": {
      backgroundColor: theme.palette.primary.main,
      "& .MuiListItemIcon-root, & .MuiListItemText-primary": {
        color: theme.palette.common.white
      }
    }
  }
}))(MenuItem);
/*******************Menu Item*******************************/

function LogoLink(props) {
  const classes = useStyles();
  const isLoggedIn = props.isLoggedIn;
  
  if (!isLoggedIn) {
    return (
      <Link to="/">
        <img className={classes.logo} src={Logo} alt="e-Valuate" />
      </Link>
    );
  } else {
    return (
      <Link to="/dashboard">
        <img className={classes.logo} src={Logo} alt="e-Valuate" />
      </Link>
    );
  }
}

function AuthButtons(props) {
  const isLoggedIn = props.isLoggedIn;
  const [anchorEl, setAnchorEl] = React.useState(null);
  let userContext = useContext(UserContext);

  const handleClick = event => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  if (!isLoggedIn) {
    return (
      <div>
        <Link to="/login">
          <LoginButton></LoginButton>
        </Link>
        <Link to="/createaccount">
          <SignUpButton></SignUpButton>
        </Link>
      </div>
    );
  } else {
    return (
      <div>
        <PrimaryButton
          onClick={handleClick}
        >
          {props.name}
          <ArrowDropDownIcon />
        </PrimaryButton>
        <StyledMenu
          id="customized-menu"
          anchorEl={anchorEl}
          keepMounted
          open={Boolean(anchorEl)}
          onClose={handleClose}
        >
          <StyledMenuItem>
            <Link to="/dashboard">
              <ListItemText primary="Dashboard" />
            </Link>
          </StyledMenuItem>
          <StyledMenuItem>
            <Link to="/editaccount">
              <ListItemText primary="Account" />
            </Link>
          </StyledMenuItem>
          <StyledMenuItem>
            <Link to="/login" onClick={userContext.logout}>
              <ListItemText primary="Logout"/>
            </Link>
          </StyledMenuItem>
        </StyledMenu>
      </div>
    );
  }
}

export default function Layout(props) {
  const classes = useStyles();
  let [tried, setTried] = React.useState(false);
  
  //get context:
  let userContext = useContext(UserContext);
  let auth = !(userContext.name == null);
  if (!tried && userContext.name == null) {
    userContext.getName();
    setTried(true);
  }

  return (
    <div className={classes.root}>
      <AppBar position="fixed" className={classes.navbar}>
        <Toolbar>
          <LogoLink isLoggedIn={auth}/>
          <div className={classes.register}>
            <AuthButtons isLoggedIn={auth} name={userContext.name}/>
          </div>
        </Toolbar>
      </AppBar>
      <div>{props.children}</div>
      <footer className={classes.footer}>
        &copy;{new Date().getFullYear()}, e-Valuate
      </footer>
    </div>
  );
}
