import React from "react";
import Layout from "./Layout"
import {Container} from '@material-ui/core'
import EvaluateHelmet from "./EvaluateHelmet";

/**
 * When using screens, you most likely should use StateScreen and let it handle
 * which screen to show. However, the other screens are exported just in case. 
 */

/**
 * The first line of a render function should be conditionally returning
 * this component if the state of loading does not equal 1. 
 * 
 * @param {number} state 
 *      0 - if currently loading
 *      1 - if loading completed with no errors
 *      All numbers greater than 1 are treated as error codes.
 */
function StateScreen(props) {
    switch(props.state) {
        case -1:
            return (<SubmittingScreen></SubmittingScreen>);
        case 0:
            return (<LoadingScreen></LoadingScreen>);
        default:
            return (<ErrorScreen errNo={props.state}></ErrorScreen>);
    }
}

function LoadingScreen(props) {
    return (
        <Layout>
            <EvaluateHelmet title="Loading"></EvaluateHelmet>
            <Container className="main">
                <h1>Loading...</h1>
            </Container>
        </Layout>
    );
}

function SubmittingScreen(props) {
    return (
        <Layout>
            <Container className="main">
                <h1>Submitting...</h1>
            </Container>
        </Layout>
    );
}

function ErrorScreen(props) {
    let errNo = props.errNo ? props.errNo : 404;
    let msg;
    switch(errNo) {
        case 404:
            msg = 'Entity Not Found';
            break;
        case 500:
            msg = 'Internal Server Error';
            break;
        default:
            msg = 'Something went wrong'
            break;
    }

    return (
        <Layout>
            <EvaluateHelmet title={`Error ${errNo}`}></EvaluateHelmet>
            <Container className="main">
                <h1>Error {errNo}</h1>
                <p>{msg}</p>
            </Container>
        </Layout>
    );
}

export { StateScreen, LoadingScreen, SubmittingScreen, ErrorScreen }
