import React, {Component} from 'react'
import EvaluateHelmet from "../components/EvaluateHelmet";
import Layout from '../components/Layout';
import { StateScreen } from '../components/Screens';
import {Container} from '@material-ui/core'
import "../static/styles/containers.scss";
import "../static/styles/forms.scss";
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { Link } from "react-router-dom";
import {PrimaryButton, AddButton, DeleteButton, EditButton} from "../components/Buttons";
import {get} from "../Utils";

function GroupRow(props) {
    let groupLetter = props.groupNumber + 65;
    if ( groupLetter === 64) {
        groupLetter = "none";
    } else {
        groupLetter = String.fromCharCode(groupLetter);
    }

    let editButton = null;
    if (props.editLink) {
        editButton = (<EditButton link={props.editLink}></EditButton>);
    }

    return (
        <TableRow>
            <TableCell>
                {props.content} {editButton}
            </TableCell>
            <TableCell align="right">{groupLetter}</TableCell>
        </TableRow>
    );
}

function InviteCodeRow(props) {
    return (
        <TableRow>
            <TableCell>{props.number}</TableCell>
            <TableCell>{props.code}</TableCell>
            <TableCell align="right">
                <DeleteButton onClick={props.remove}/>
            </TableCell>
        </TableRow>
    );
}

class EventDetail extends Component {

    constructor(props){
        super(props);
        this.state = {
            loading: 0
        }
        this.getCode = this.getCode.bind(this);
    }

    async componentDidMount() {
        //get event data:
        let {status, body} = await get(`/event/${this.props.match.params.eventId}`);

        if ( status === 200 ) {//okay
            let role = body.role;
            switch(role) {
                case "anon":
                    this.setState({
                        role,
                        judgeId: null,
                        event: body.event, 
                        inviteCodes: null, 
                        judges: null,
                        entries: null,
                        loading: 1
                    });
                    break;
                case "judge":
                    this.setState({
                        role,
                        judgeId: body.judgeId,
                        event: body.event, 
                        inviteCodes: null, 
                        judges: null,
                        entries: null,
                        loading: 1
                    });
                    break;
                case "chair":
                    this.setState({
                        role,
                        judgeId: body.judgeId,
                        event: body.event, 
                        inviteCodes: body.inviteCodes, 
                        judges: body.judges,
                        entries: body.entries,
                        loading: 1
                    });
                    break;
                default:
                    break;
            }
        } else {
            this.setState({loading: status});
        }
    }

    codeCompare(a, b) {
        return a.code.localeCompare(b.code);
    }

    async deleteCode(index) {
        let code = this.state.inviteCodes[index].code;
        let eventId = this.state.event.id;
        let response = await fetch("/event/" + eventId + "/invitecode/" + code, {
            method: "DELETE",
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json"
            }
        });
        let status = response.status;

        if (status === 200) {//okay
            let body = await response.json();
            body.sort(this.codeCompare);
            this.setState({inviteCodes: body});
        } else if ( status === 404 ) {//not found
            alert("Unable to create invite code.");
        } else if ( status === 500 ) {//internal server error
            alert("Cannot connect to server. Please try again later.");
        }
    }

    async getCode(){
        let eventId = this.state.event.id;
        let response = await fetch("/event/" + eventId + "/invitecode", {
            method: "POST",
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json"
            }
        });
        let status = response.status;

        if (status === 200) {//okay
            let body = await response.json();
            body.sort(this.codeCompare);
            this.setState({inviteCodes: body});
        } else if ( status === 404 ) {//not found
            alert("Unable to create invite code.");
        } else if ( status === 500 ) {//internal server error
            alert("Cannot connect to server. Please try again later.");
        }
    }

    buildChairTables() {
        //build invite codes table:
        let inviteCodes = this.state.inviteCodes;
        let icRows = [];
        for (let i = 0; i < inviteCodes.length; i++) {
            icRows.push(<InviteCodeRow number={i+1} code={inviteCodes[i].code} remove={() => this.deleteCode(i)}/>);
        }

        //build judges table:
        let judges = this.state.judges;
        let jRows = [];
        for (let i = 0; i < judges.length; i++) {
            jRows.push(<GroupRow content={judges[i].name} groupNumber={judges[i].groupNumber}/>);
        }

        //build entries table:
        let entries = this.state.entries;
        let eRows = [];
        for (let i = 0; i < entries.length; i++) {
            eRows.push(
                <GroupRow 
                    content={entries[i].title}
                    editLink={`/event/${this.state.event.id}/entry/${entries[i].id}`}
                    groupNumber={entries[i].groupNumber}
                />
            );
        }

        return (
            <div>
                <br />
                <br />
                <h4>Invitation Codes</h4>
                <Table id="invitesTable" size="small">
                    <TableHead>
                        <TableRow>
                            <TableCell><b>Number</b></TableCell>
                            <TableCell><b>Code</b></TableCell>
                            <TableCell align="right"><AddButton onClick={this.getCode} /></TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {icRows}
                    </TableBody>
                </Table>
                <br />
                <h4>Judges</h4>
                <Table id="judgesTable" size="small">
                    <TableHead>
                        <TableRow>
                            <TableCell><b>Name</b></TableCell>
                            <TableCell align="right"><b>Group Number</b></TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {jRows}
                    </TableBody>
                </Table>
                <br />
                <h4>Entries</h4>
                <Table id="entriesTables" size="small">
                    <TableHead>
                        <TableRow>
                            <TableCell><b>Name</b></TableCell>
                            <TableCell align="right"><b>Group Number</b></TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {eRows}
                    </TableBody>
                </Table>
            </div>
        );
    }

    render(){
        if ( this.state.loading !== 1 ) {
            return (
                <StateScreen state={this.state.loading}></StateScreen>
            );
        }

        let role = this.state.role;
        let event = this.state.event;
 
        let eventLink = null;
        let editEventButton = null;
        let rubricButton = null;
        let scoreReportButton = null;
        let extraBreak = null;
        let assignGroupsButton = null;
        if (role === "chair") {
            eventLink = (
                <p>Share this link with your contestants: {window.location.href}</p>
            );

            editEventButton = (
                <Link to={"/editEvent/" + event.id}>
                    <PrimaryButton>Edit Details</PrimaryButton>
                </Link>
            );

            let rubricType = (event.rubric === null) ? "create" : "edit";
            rubricButton = (
                <Link to={"/event/" + event.id + "/" + rubricType + "rubric"}>
                    <PrimaryButton>{rubricType} rubric</PrimaryButton>
                </Link>
            );

            scoreReportButton = (
                <Link to={"/event/" + event.id + "/scorereport"}>
                    <PrimaryButton>Score Report</PrimaryButton>
                </Link>
            );

            extraBreak = (
                <div>
                    <br />
                    <br />
                </div>
            );

            assignGroupsButton = (
                <Link to={"/event/" + event.id + "/assigngroups"}>
                    <PrimaryButton>Assign Groups</PrimaryButton>
                </Link>
            );
        }

        let assignedEntriesButton = null;
        if (role === "chair" || role === "judge") {
            assignedEntriesButton = (
                <Link to={"/event/" + event.id + "/judge/" + this.state.judgeId + "/assignedentries"}>
                    <PrimaryButton>Assigned Entries</PrimaryButton>
                </Link>
            );
        }

        let chairTables = null;
        if (role === "chair") {
            chairTables = this.buildChairTables();
        }

        let registerButton = null;
        if (role === "anon") {
            registerButton = (
            <Link to={"/event/" + event.id + "/eventregister"}>
                YOU DO NOT NEED AN ACCOUNT TO REGISTER!
                <br /><br />
                <PrimaryButton>Register</PrimaryButton>
            </Link>
            );
        }

        return (
            <Layout>
            <EvaluateHelmet title="Event Details"></EvaluateHelmet>
            <Container className="main">
                <div className="container">
                    <form id="eventDetailForm">
                        <h2>{event.name}</h2>
                        <br />
                        <h4>Event Time: {event.date + " " + event.time}</h4>
                        <br />
                        <h4>Event Location: {event.location}</h4>
                        <br />
                        <h4>Event Description: {event.description}</h4>
                        {eventLink}
                        <br />
                        {registerButton}
                        {editEventButton}
                        {rubricButton}
                        {scoreReportButton}
                        {extraBreak}
                        {assignedEntriesButton}
                        {chairTables}
                        <br />
                        {assignGroupsButton}
                    </form>
                </div>
            </Container>
            </Layout>
        );
    }
}

export default EventDetail;
