import React, { Component } from "react";
import EvaluateHelmet from "../../components/EvaluateHelmet";
import Layout from '../../components/Layout';
import TextFields from '../../components/TextFields';
import { PrimaryButton } from "../../components/Buttons";
import { Link } from "react-router-dom";
import { Container, Grid } from '@material-ui/core';
import { StateScreen } from '../../components/Screens';
import {get} from '../../Utils';

class CreateAccount extends Component {

  emptyAccount = {
    firstName: "",
    lastName: "",
    email: "",
    password: "",
    encryptedPassword: "",
    confirmPassword: ""
  };

  emptyIsValid = {
    firstName: false,
    lastName: false,
    email: false,
    password: false,
    confirmPassword: false
  };

  constructor(props) {
    super(props);
    this.state = {
      loading: 0,
      editMode: false,
      account: this.copy(this.emptyAccount),
      error: this.copy(this.emptyAccount),
      isValid: this.copy(this.emptyIsValid)
    }
    this.handleChange = this.handleChange.bind(this);
  }

  async componentDidMount() {
    let editRegex = /\/edit/;
    if ( editRegex.test(this.props.location.pathname) ) {//if editting
      let {status, body} = await get(`/register/edit`);
      if (status === 200) {
        let isValid = this.state.isValid;
        delete isValid.confirmPassword;
        isValid.firstName = true
        isValid.lastName = true
        isValid.email = true
        this.setState({
          editMode: true,
          isValid,
          account: body,
          loading: 1
        });
      } else {
        this.setState({loading: status});
      }
    } else {
      this.setState({loading: 1});
    }
  }

  copy(object) {
    return JSON.parse(JSON.stringify(object));
  }

  /*
   * If all the input is correct then returns changed as flase, otherwise changed is true
   */
  async validate(name) {
    //Following line contain warnings. Please ignore.
    let emailRegEx = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,5})+$/;
    let nameRegEx = /^([^\d\W]+\s?)+$/;//one or more words without digits
    let ht = "";
    let v = true;
    let account = this.state.account;

    switch(name) {
      case "firstName":
        if (!nameRegEx.test(account.firstName)) {
          ht = "Invalid first name.";
          v = false;
        }
        break;
      case "lastName":
        if (!nameRegEx.test(account.lastName)) {
          ht = "Invalid last name.";
          v = false;
        }
        break;
      case "email":
        if (!emailRegEx.test(account.email)) {
          ht = "Invalid email.";
          v = false;
        }
        break;
      case "password":
        if (account.password.length < 6 || account.password.length > 25) {
          ht = "Must be between 6 and 25 characters.";
          v = false;
        }
        break;
      case "confirmPassword":
        if (account.confirmPassword !== account.password) {
          ht = "Passwords do not match.";
          v = false;
        }
        break;
      default:
        break;
    }

    let error = this.state.error;
    let isValid = this.state.isValid;
    error[name] = ht;
    isValid[name] = v;
    this.setState({ error: error, isValid: isValid });
  };

  async handleSubmit() {//TODO check updateAccount in backend
    this.setState({loading: -1});
    let {account, editMode, isValid} = this.state;
    account.encryptedPassword = account.password;
    let uri = editMode ? '/register/edit' : '/register';
    let response = await fetch(uri,
      {
        method: editMode ? 'PUT' : 'POST',
        headers: {
          "Accept": "application/json",
          "Content-Type": "application/json"
        },
        body: JSON.stringify(account)
      }
    );

    if ( response.status === 200 ) {//okay
      this.props.history.push('/dashboard');
    } if ( response.status === 201 ) {//created
      this.props.history.push('/login');
    }
    else if (response.status === 403) {//forbidden
      let msg = editMode ? "Password is incorrect." : "Password must be between 6 and 25 characters.";
      alert(msg);
    }
    else if (response.status === 409) {//conflict
      let msg = "An account already exists for this email.";
      msg += editMode ? "" : " You may reset your password on the following page.";
      alert(msg);
      if (!editMode) {
        this.props.history.push('/forgotpassword');
      }
    }
    account = this.state.account;
    isValid.password = false;
    account.password = "";
    this.setState({loading: 1, account, isValid});
  };

  async handleChange(e) {
    e.persist();
    const value = e.target.value;
    const id = e.target.id;
    let account = this.state.account;
    account[id] = value;
    this.setState({ account: account });
  }

  render() {
    if ( this.state.loading !== 1 ) {
      return (
        <StateScreen state={this.state.loading}></StateScreen>
      );
    }

    let {account, editMode} = this.state;

    let isDisabled = null;
    let isValid = this.state.isValid;
    for (var key in isValid) {
      if (isValid.hasOwnProperty(key) && !isValid[key] ) {
        isDisabled = true;
        break;
      }
    }

    let msg;
    if (editMode) {
      msg = (
        <div>
          <p>To edit your account information, please verify you current password.</p>
          <p>You may not change your password from this page.</p>
        </div>
      );
    } else {
      msg = (
        <Link to="/login">
          Already have an account? Log in
        </Link>
      );
    }

    return (
      <Layout>
        <EvaluateHelmet title={editMode ? "Edit Account Info" : "Create an Account"}></EvaluateHelmet>
        <Container maxWidth="sm" className="main">
          <h1>Create an Account</h1>
          <Grid container spacing={2}>
            <Grid item xs={12} sm={6}>
              <TextFields
                autoFocus
                label="First Name"
                name="firstName"
                id="firstName"
                type="text"
                required
                fullWidth
                defaultValue={account.firstName}
                onChange={this.handleChange}
                helperText={this.state.error.firstName}
                onBlur={() => this.validate("firstName")}
              />
            </Grid>
            <Grid item xs={12} sm={6}>
              <TextFields
                label="Last Name"
                name="lastName"
                id="lastName"
                type="text"
                required
                fullWidth
                defaultValue={account.lastName}
                onChange={this.handleChange}
                helperText={this.state.error.lastName}
                onBlur={() => this.validate("lastName")}
              />
            </Grid>
            <Grid item xs={12}>
              <TextFields
                label="Email"
                name="email"
                id="email"
                type="text"
                required
                fullWidth
                defaultValue={account.email}
                onChange={this.handleChange}
                helperText={this.state.error.email}
                onBlur={() => this.validate("email")}
              />
            </Grid>
            <Grid item xs={12}>
              <TextFields
                label="Password"
                name="password"
                id="password"
                type="password"
                required
                fullWidth
                onChange={this.handleChange}
                helperText={this.state.error.password}
                onBlur={() => this.validate("password")}
              />
            </Grid>
            {!editMode &&
            <Grid item xs={12}>
              <TextFields
                label="Confirm Password"
                name="confirmPassword"
                id="confirmPassword"
                type="password"
                required
                fullWidth
                onChange={this.handleChange}
                helperText={this.state.error.confirmPassword}
                onBlur={() => this.validate("confirmPassword")}
              />
            </Grid>}
            <Grid item xs={12}>
              <PrimaryButton
                disabled={isDisabled} 
                fullWidth
                onClick={() => this.handleSubmit()}
              >
                {editMode ? "Save Changes" : "Sign Up"}
              </PrimaryButton>
            </Grid>
            <Grid item>
              {msg}
            </Grid>
          </Grid>
        </Container>
      </Layout>
    );
  }
}

export default CreateAccount;