import React, { Component } from 'react';
import EvaluateHelmet from "../../components/EvaluateHelmet";
import TextFields from '../../components/TextFields';
import { Button, Container, Grid } from '@material-ui/core';
import Layout from '../../components/Layout';
import { PrimaryButton } from '../../components/Buttons';
import { StateScreen } from '../../components/Screens';
import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions
} from "@material-ui/core";

class ForgotPassword extends Component {
  
  constructor(props) {
    super(props);
    this.state = {
      email: "",
      loading: 1,
      popup: {
        isOpen: false
      }
    };

    this.handleChange = this.handleChange.bind(this);
  }

  handleChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  async handleSubmit() {
    this.setState({loading: -1});
    let account = {};
    account.email = this.state.email;
    
    let response = await fetch('/reset/forgot', 
      {
        method: "POST",
        headers: {
          "Accept": "application/json",
          "Content-Type": "application/json"
        },
        body: JSON.stringify(account)
      }
    );

    if ( response.status === 200 ) {//okay
      let popup = this.state.popup;
      popup.isOpen = true;
      this.setState({popup: popup});
    } else if( response.status === 500 ) {//internal server error
      alert("Could not connect to server. Please try again later.");
    }
    this.setState({loading: 1});
  }
  
  render() {
    if ( this.state.loading !== 1 ) {
      return (
        <StateScreen state={this.state.loading}></StateScreen>
      );
    }

    //set up popup:
    const popup = this.state.popup;
    const dialogClose = () => {
      var popup = this.state.popup;
      popup.isOpen = false;
      this.setState({ popup: popup });
    };

    return (
      <Layout>
        <EvaluateHelmet title="Forgot Password"></EvaluateHelmet>
        <Container component="main" maxWidth="sm" className="main">
          <h1>Forgot Password</h1>
          <Grid container spacing={2}>
            <Grid item xs={12}>
              <p>
                If you forgot the password, please enter the account's email below.
              </p>
            </Grid>
            <Grid item xs={12}>
              <TextFields
                margin="normal"
                fullWidth
                id="email"
                label="Email"
                name="email"
                autoComplete="email"
                onChange={(e) => this.handleChange(e)}
                autoFocus
              />
            </Grid>
            <Grid item xs={12}>
              <PrimaryButton
                onClick={() => this.handleSubmit()}
                fullWidth
              >
                Send Email
              </PrimaryButton>
            </Grid>
          </Grid>
          <Dialog
            open={popup.isOpen}
            onClose={dialogClose}
            aria-labelledby="alert-dialog-slide title"
          >
            <DialogTitle id="alert-dialog-slide title">
              Email Sent
            </DialogTitle>
            <DialogContent>
              If you have an account with this email, you will receive 
              an email within the next 10 to 15 minutes with instructions
              on how to reset your password. 
              <br />
              If you do not receive an email, please submit again. 
              If you submit multiple times, only the last email will 
              allow you to reset your password.
            </DialogContent>
            <DialogActions>
              <Button color="primary" onClick={dialogClose}>
                Okay
              </Button>
            </DialogActions>
          </Dialog>
        </Container>
      </Layout>
    );
  }

}
                
export default ForgotPassword;
