import React, { Component } from "react";
import EvaluateHelmet from "../../components/EvaluateHelmet";
import TextFields from "../../components/TextFields";
import { Link } from "react-router-dom";
import { Container, Grid } from "@material-ui/core";
import Layout from "../../components/Layout";
import { PrimaryButton } from "../../components/Buttons";

class Login extends Component {

  constructor(props) {
    super(props);
    this.state = {
      props: props,
      email: "",
      password: ""
    };

    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  async handleSubmit(event) {
    event.preventDefault();
    let account = this.state;
    account.encryptedPassword = account.password;
    let response = await fetch("/login", {
      method: "POST",
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      body: JSON.stringify(account)
    });
    if (response.status === 200) {
      this.state.props.history.push("/dashboard");
    } else if (response.status === 500) {
      alert("Could not connect to server. Please try again later.");
    } else {
      alert("Incorrect email or password.");
    }
    return false;
  }

  render() {
    let isDisabled = true;
    if ( this.state.email.length !== 0 && this.state.password.length !== 0 ) {
      isDisabled = null
    }

    return (
      <Layout>
        <EvaluateHelmet title="Log In"></EvaluateHelmet>
        <Container component="main" maxWidth="sm" className="main">
          <h1>Log In</h1>
          <form onSubmit={this.handleSubmit}>
          <Grid container spacing={2}>
            <Grid item xs={12}>
              <TextFields
                margin="normal"
                fullWidth
                id="email"
                label="Email"
                name="email"
                autoComplete="email"
                onChange={this.handleChange}
                autoFocus
              />
            </Grid>
            <Grid item xs={12}>
              <TextFields
                margin="normal"
                fullWidth
                name="password"
                label="Password"
                type="password"
                id="password"
                autoComplete="current-password"
                onChange={this.handleChange}
              />
            </Grid>
            <Grid item xs={12}>
              <PrimaryButton
                disabled={isDisabled}
                fullWidth
                type="submit"
              >
                Log In
              </PrimaryButton>
            </Grid>
            <Grid item sm={6}>
              <Link to="/forgotpassword">
                {"Forgot your password? Reset it"}
              </Link>
            </Grid>
            <Grid item sm={6}>
              <Link to="/createaccount">
                {"Don't have an account? Create one"}
              </Link>
            </Grid>
          </Grid>
          </form>
        </Container>
      </Layout>
    );
  }
}

export default Login;
