import React, { Component } from 'react';
import EvaluateHelmet from "../../components/EvaluateHelmet";
import TextFields from '../../components/TextFields';
import { Container, Grid } from '@material-ui/core';
import Layout from '../../components/Layout';
import { PrimaryButton } from '../../components/Buttons';

class ResetPassword extends Component {
  
  constructor(props) {
    super(props);
    this.state = {
      code: props.match.params.code,
      email: "",
      password: "",
      confirmPassword: "",
      helperText: {
        email: "",
        password: "",
        confirmPassword: ""
      },
      isValid: {
        email: false,
        password: false,
        confirmPassword: false
      }
    };

    this.handleChange = this.handleChange.bind(this);
  }

  async handleChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  async handleSubmit() {
    let account = {
      email: this.state.email,
      encryptedPassword: this.state.password
    };
    
    let response = await fetch('/reset/code/' + this.state.code, 
      {
        method: "POST",
        headers: {
          "Accept": "application/json",
          "Content-Type": "application/json"
        },
        body: JSON.stringify(account)
      }
    );

    if ( response.status === 200 ) {//okay
      this.props.history.push("/login");
    } else if ( response.status === 404 ) {//not found
      alert("Unable to reset password. Link or email is incorrect.");
    } else if( response.status === 500 ) {//internal server error
      alert("Could not connect to server. Please try again later.");
    }
  }

  async validate(name) {
    let ht = "";
    let v = true;
    let password = this.state.password;

    switch(name) {
      case "email":
        let email = this.state.email;
        let emailRegEx = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,5})+$/;
        if (!emailRegEx.test(email)) {
          ht = "Must be a valid email address.";
          v = false;
        }
        break;
      case "password":
        if (password.length < 6 || password.length > 20) {
          ht = "Must be between 6 and 20 characters.";
          v = false;
        }
        break;
      case "confirmPassword":
        let confirmPassword = this.state.confirmPassword;
        if (confirmPassword !== password) {
          ht = "Passwords do not match.";
          v = false;
        }
        break;
      default:
        break;
    }

    let helperText = this.state.helperText;
    let isValid = this.state.isValid;
    helperText[name] = ht;
    isValid[name] = v;

    this.setState({helperText: helperText, isValid: isValid});
  }
  
  render() {
    //check validatiy:
    let isValid = this.state.isValid;
    let isDisabled = null;
    if ( !isValid.email ) {
      isDisabled = true;
    } else if ( !isValid.password ) {
      isDisabled = true;
    } else if ( !isValid.confirmPassword ) {
      isDisabled = true;
    }

    return (
      <Layout>
        <EvaluateHelmet title="Reset Password"></EvaluateHelmet>
        <Container component="main" maxWidth="sm" className="main">
          <h1>Reset Password</h1>
          <Grid container spacing={2}>
            <Grid item xs={12}>
              <TextFields
                margin="normal"
                fullWidth
                id="email"
                label="Email"
                name="email"
                autoComplete="email"
                helperText={this.state.helperText.email}
                onChange={(e) => this.handleChange(e)}
                onBlur={() => this.validate("email")}
                autoFocus
              />
            </Grid>
            <Grid item xs={12}>
              <TextFields
                label="New Password"
                name="password"
                id="password"
                type="password"
                fullWidth
                helperText={this.state.helperText.password}
                onChange={(e) => this.handleChange(e)}
                onBlur={() => this.validate("password")}
              />
            </Grid>
            <Grid item xs={12}>
              <TextFields
                label="Confirm Password"
                name="confirmPassword"
                id="confirmPassword"
                type="password"
                fullWidth
                helperText={this.state.helperText.confirmPassword}
                onChange={(e) => this.handleChange(e)}
                onBlur={() => this.validate("confirmPassword")}
              />
            </Grid>
            <Grid item xs={12}>
              <PrimaryButton
                onClick={() => this.handleSubmit()}
                fullWidth
                disabled={isDisabled}
              >
                Reset Password
              </PrimaryButton>
            </Grid>
          </Grid>
        </Container>
      </Layout>
    );
  }

}
                
export default ResetPassword;
