import React, { Component } from "react";
import EvaluateHelmet from "../../components/EvaluateHelmet";
import Layout from "../../components/Layout";
import { StateScreen } from '../../components/Screens';
import { PrimaryButton } from "../../components/Buttons";
import { Container, TextField } from "@material-ui/core";
import "../../static/styles/containers.scss";
import "../../static/styles/forms.scss";
import {
  MuiPickersUtilsProvider,
  KeyboardDateTimePicker
} from "@material-ui/pickers";
import DateFnsUtils from "@date-io/date-fns";
import {get} from "../../Utils";

class CreateEvent extends Component {
  emptyEvent = {
    name: "",
    description: "",
    dateTime: new Date(),
    location: ""
  };

  constructor(props) {
    super(props);
    this.state = this.emptyEvent;
    this.state.loading = 0;
    this.state.isModifying = false;
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleDateChange = this.handleDateChange.bind(this);
  }

  async componentDidMount() {
    //check if editing existing event
    if ( 'eventId' in this.props.match.params ) {
      let {status, body} = await get(`/createevent/${this.props.match.params.eventId}`);

      if ( status === 200) {//okay
        this.setState({
          isModifying: true,
          id: body.id,
          name: body.name,
          description: body.description,
          dateTime: body.dateTime,
          location: body.location,
          loading: 1
        });
      } else {
        this.setState({loading: status});
      }
    } else {
      this.setState({loading: 1});
    }
  }

  handleChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  handleDateChange(dateTime) {
    this.setState({ dateTime });
  }

  /**
   * Posts the new event.
   */
  async handleSubmit(e) {
    e.preventDefault();
    var event = this.state;

    //decide method:
    let method = (this.state.isModifying) ? "PUT" : "POST";

    //post or put event:
    let response = await fetch("/createevent", {
      method: method,
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      body: JSON.stringify(event)
    });

    let body = await response.json();

    if (response.status === 201) {
      this.props.history.push("/event/" + body.id);//TODO remove state
    } else if( response.status === 500 ) {
      alert("Could not connect to server. Please try again later.");
    } else {
      alert("Creation unsuccessful. Please try again.");
    }
  }

  render() {
    if ( this.state.loading !== 1 ) {
        return (
            <StateScreen state={this.state.loading}></StateScreen>
        );
    }

    //set title text:
    let title = this.state.isModifying ? "Edit" : "Create";
    title += " an Event";

    //set button text:
    let buttonText = (this.state.isModifying) ? "Save Changes" : "Create";

    return (
      <Layout>
        {/* Helmet is for the <head> */}
        <EvaluateHelmet title={title}></EvaluateHelmet>
        <Container className="flexDisplay main">
          <div>
            <h1>{title}</h1>
            <form
              onSubmit={event => {
                this.handleSubmit(event);
              }}
              id="eventCreationForm"
            >
              <TextField
                id="event_title"
                label="Event Title"
                name="name"
                fullwidth
                onChange={this.handleChange}
                value={this.state.name}
              />
              <br />
              <MuiPickersUtilsProvider utils={DateFnsUtils}>
                <KeyboardDateTimePicker
                  id="date-time-picker"
                  label="Event Date & Time"
                  format="yyyy/MM/dd hh:mm a"
                  value={this.state.dateTime}
                  onChange={this.handleDateChange}
                  KeyboardButtonProps={{
                    "aria-label": "change time"
                  }}
                />
              </MuiPickersUtilsProvider>
              <br />
              <TextField
                id="event_location"
                label="Event Location"
                name="location"
                fullwidth
                onChange={this.handleChange}
                value={this.state.location}
              />
              <br />
              <TextField
                id="des"
                label="Description"
                name="description"
                multiline
                fullwidth
                rows="4"
                onChange={this.handleChange}
                value={this.state.description}
              />
              <br />
              <PrimaryButton type="submit">{buttonText}</PrimaryButton>
              <br />
            </form>
          </div>
        </Container>
      </Layout>
    );
  }
}

export default CreateEvent;
