import React, { Component } from "react";
import Layout from "../../components/Layout";
import { StateScreen } from '../../components/Screens';
import EvaluateHelmet from "../../components/EvaluateHelmet";
import {
  Container,
  Grid,
  Card,
  Table,
  TableCell,
  TableBody,
  TableContainer,
  TableHead,
  TableRow,
  Typography,
} from "@material-ui/core";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import {
  faFileImport,
  faUserCheck
} from "@fortawesome/free-solid-svg-icons";
import {ExportXLSX} from '../../components/ExportXLSX'
import {EditButton, ExpandButton, PrimaryButton} from '../../components/Buttons'
import EmptyCard from '../../components/EmptyCard'
import {get} from '../../Utils'

function GroupCard(props) {
  return (
    <Card className="mt-4" >
      <Typography variant="h4" className="text-center">
        GROUP {props.groupLetter}
      </Typography>
      <TableContainer>
      <Table stickyHeader aria-label="sticky table" >
        <TableHead>
          <TableRow>
            <TableCell></TableCell>
            <TableCell>ENTRY TITLE</TableCell>
            <TableCell className="text-center">TOTAL SCORE</TableCell>
            <TableCell className="text-center">AVERAGE SCORE</TableCell>
          </TableRow>
        </TableHead>
        <TableBody>
          {props.entryList}
        </TableBody>
      </Table>
      </TableContainer>
    </Card>
  );
}

function EntryRow(props) {
  let entry = props.entry;
  return (
    <TableRow>
      <TableCell>
        <ExpandButton 
          isShowing={entry.isShowing}
          onClick={props.showDetails}
          color="primary"
        ></ExpandButton>
      </TableCell>
      <TableCell>
        <EditButton link={`/event/${props.eventId}/entry/${props.entry.id}`} ></EditButton>
        {entry.title}
      </TableCell>
      <TableCell className="text-center">
        {entry.total}
      </TableCell>
      <TableCell className="text-center">
        {entry.average}
      </TableCell>
    </TableRow>
  );
}

function ScoredRubricTable(props) {
  let {entry, catList, scoredRubricRows} = props;
  return (
    <TableRow id={entry+"ID"}>
      <TableCell colSpan={5}>
        <TableContainer>
          <Table stickyHeader aria-label="sticky table">
            <TableHead>
              <TableRow>
                <TableCell className="text-center">JUDGE NAME</TableCell>
                {catList.map((cat) => (
                  <TableCell className="text-center" key={cat.id}>{cat.name}</TableCell>
                ))}
                <TableCell className="text-center">TOTAL SUBSCORE</TableCell>
                <TableCell></TableCell>
              </TableRow>
            </TableHead>
            <TableBody>
              {scoredRubricRows}
            </TableBody>
          </Table>
        </TableContainer>
      </TableCell>
    </TableRow>
  );
}

function ScoredRubricRow(props) {
  let {scoredRubric, judge, link} = props;

  let cells = [];
  for (let s in scoredRubric.scores) {
    cells.push(
      <TableCell className="text-center" key={s}>{scoredRubric.scores[s]}</TableCell>
    );
  }

  return (
    <TableRow>
      <TableCell>{judge}</TableCell>
      {cells}
      <TableCell className="text-center">{scoredRubric.total}</TableCell>
      <TableCell>
        <EditButton link={link} color="secondary"></EditButton>
      </TableCell>
    </TableRow>
  );
}

class ScoreReport extends Component {

  constructor(props) {
    super(props);
    this.state = {
      catList: [],
      groups: [],
      eventInfo: {},
      loading: 0
    };
  }

  async componentDidMount() {
    let {status, body} = await get(`/event/${this.props.match.params.eventId}/scorereport`);
    if (status === 200) {
      this.pruneData(body);
    } else {
      this.setState({loading: status});
    }
  }

  getJudgeName(judgeId, judges) {
    for(let judge of judges){
      var roles = judge.roles
      for(let role of roles){
        if(role.id === judgeId){
          return judge.fullName;
        }
      }
    }
    return null;
  }

  scoredEntries() {
    const { catList, eventInfo, groups } = this.state;
    let eventId = eventInfo.id;

    let groupCards = [];
    let numEntries = 0;
    if(groups.length === 0){
      groupCards.push(
        <EmptyCard text="No data is available."></EmptyCard>
      )
    }

    for(let g in groups){// iterate through groups
      let group = groups[g];
      let groupLetter = String.fromCharCode(parseInt(g) + 65);
      let entryRows = [];
      for (let e in group){//iterate through entries within a group
        let entry = group[e];
        let scoredRubricRows = [];

        entryRows.push(
          <EntryRow 
            eventId={eventId}
            entry={entry} 
            showDetails={() => this.showDetails(g, e)}
            key={entry.title}
          />
        );
        
        if (entry.isShowing) {
          for (let i in entry.scoredRubrics) {
            let sr = entry.scoredRubrics[i];
            let link = `/event/${eventInfo.id}/chair/${sr.judge.id}/entry/${entry.id}/gradeentry`;
            scoredRubricRows.push(
              <ScoredRubricRow 
                scoredRubric={sr}
                judge={sr.judge.name}
                link={link}
                key={i}
              />
            );
          }

          entryRows.push(
            <ScoredRubricTable 
              entry={entry.title}
              catList={catList}
              scoredRubricRows={scoredRubricRows}
              key={entry.title+"score"}
            />
          );
        }

        numEntries++;
      }
      
      groupCards.push(
        <GroupCard group={group} groupLetter={groupLetter} entryList={entryRows} key={g}/>
      );
    }

    return { groupCards, numEntries };
  }

  showDetails(g, e) {
    let groups = this.state.groups;
    groups[g][e].isShowing = !groups[g][e].isShowing;
    this.setState({groups});
  }

  sortEntries(a, b) {
    return b.total - a.total;
  }

  sortScoredRubrics(a, b) {
    return a.judge.name.localeCompare(b.judge.name);
  }

  pruneData(body) {
    let groups = [];
    for (let groupNum in body.scoredEntries) {//iterate through groups
      let group = [];// list of entries in group
      
      for (let entryTitle in body.scoredEntries[groupNum]) {//iterate through entries
        let scoredRubrics = body.scoredEntries[groupNum][entryTitle];
        if(scoredRubrics.length > 0){
          let entry = {};// an entry's info and its list of scoredRubrics
          entry.title = entryTitle;
          entry.id = scoredRubrics[0].entry.id;
          entry.total = 0;
          entry.isShowing = false;
          entry.scoredRubrics = [];
          for (let i in scoredRubrics) {//iterate through scoredRubrics
            let oldSr = scoredRubrics[i];
            let scoredRubric = {};// a scoredRubric's judge info and scores
            scoredRubric.judge = {
              name: this.getJudgeName(oldSr.judge.id, body.judges),
              id: oldSr.judge.id
            };
            scoredRubric.judgeNote = oldSr.judgeNote;
            scoredRubric.scores = [];// a list of scores containing no sub scores
            scoredRubric.total = 0;

            for (let k in oldSr.scoredCategories) {//iterate through scores
              scoredRubric.scores.push(oldSr.scoredCategories[k].pointsAwarded);
              scoredRubric.total += scoredRubric.scores[k];
            }

            entry.total += scoredRubric.total;
            entry.scoredRubrics.push(scoredRubric);
          }

          entry.average = (entry.total / entry.scoredRubrics.length).toFixed(2);
          entry.scoredRubrics.sort(this.sortScoredRubrics);
          group.push(entry);
        }
      }
      groups.push(group.sort(this.sortEntries));
    }

    this.setState({
      catList: body.catList,
      groups: groups,
      eventInfo: body.eventInfo,
      numJudges: body.judges.length,
      loading: 1
    });
  }

  render() {
    if ( this.state.loading !== 1 ) {
      return (
          <StateScreen state={this.state.loading}></StateScreen>
      );
    }

    const { catList, numJudges, eventInfo, groups } = this.state;

    const se = this.scoredEntries();

    return (
      <Layout>
        <EvaluateHelmet title="Score Report"></EvaluateHelmet>
        <Container className="main">
          <Grid className="mt-4" container spacing={3}>
          <Grid item md={12} xs={12}>
              <Card className="p-2">
                <Typography style={{marginBottom: '25px'}} variant="h3">{eventInfo.name}</Typography>
                {groups.length === 0 ? "" : (<ExportXLSX categoryList={catList} csvData={groups} fileName={eventInfo.name}></ExportXLSX>)}
              </Card>
            </Grid>
          </Grid>
          <Grid className="mt-4" container spacing={3}>
            <Grid item md={6} xs={12}>
              <Card className="p-2">
                <p style={{ color: "gray" }}>
                  <FontAwesomeIcon icon={faFileImport} /> Graded Entries:
                </p>
                <h1 className="text-center">
                  {se.numEntries}
                </h1>
              </Card>
            </Grid>
            <Grid item md={6} xs={12}>
              <Card className="p-2">
                <p style={{ color: "gray" }}>
                  <FontAwesomeIcon icon={faUserCheck} /> Assigned Judges:
                </p>
                <h1 className="text-center">{numJudges}</h1>
              </Card>
            </Grid>
          </Grid>
          <div>{se.groupCards}</div>
        </Container>
      </Layout>
    );
  }
}

export default ScoreReport;
