import React, { Component } from 'react'
import EvaluateHelmet from "../../components/EvaluateHelmet";
import { Link } from "react-router-dom";
import Layout from '../../components/Layout';
import { StateScreen } from '../../components/Screens';
import { Container, Card, CardContent, Typography, CardHeader } from '@material-ui/core'
import { makeStyles } from '@material-ui/core/styles';
import EmptyCard from '../../components/EmptyCard'
import { get } from '../../Utils';

const useStyles = makeStyles({
    card: {
        maxWidth: "100%",
        height: "50%"
    },
});

function EntryCard(props) {
    // Create the link to the Grade an Entry site
    const gradingUrl = "/event/" + props.eventId + "/judge/" + props.judgeId
        + "/entry/" + props.entry.id + "/gradeentry";

    const classes = useStyles();
    let score = props.entry.currentPoints;

    return (
        <Link to={gradingUrl}>
            <Card className={classes.card}>
                <CardHeader
                    title={props.entry.title}
                    action={<h5>{score + " points"}</h5>}
                />
                <CardContent>
                    <Typography variant="inherit" display="inline">
                        {props.entry.description}
                    </Typography>
                </CardContent>
            </Card>
            <br></br>
        </Link>
    );
}

class AssignedEntries extends Component {

    constructor(props) {
        super(props)

        this.state = {
            eventId: props.match.params.eventId,
            judgeId: props.match.params.judgeId,
            entries: [],
            loading: 0
        }
    }

    async componentDidMount() {
        const {eventId, judgeId} = this.state;
        let {status, body} = await get(`/event/${eventId}/judge/${judgeId}/assignedentries`);
        if (status === 200) {//okay
            // Get List of Assign Entries:
            let entries = body.assignedEntries;
            let scores = body.scores;

            // Attribute scores to entries:
            for (let i = 0; i < entries.length; i++) {
                entries[i].currentPoints = scores[i];
            }
            this.setState({entries: entries, loading: 1})
        } else {
            this.setState({loading: status});
        }
    }

    render() {
        if ( this.state.loading !== 1 ) {
            return (
                <StateScreen state={this.state.loading}></StateScreen>
            );
        }

        var entryCards = [];
        const entries = this.state.entries;
        if(entries.length === 0){
            entryCards.push(
                <EmptyCard text="You don't have any entry."></EmptyCard>
            )
        }
        for (var i = 0; i < entries.length; i++) {
            entryCards.push(
                <EntryCard entry={entries[i]} judgeId={this.state.judgeId} eventId={this.state.eventId} />
            )
        }

        return (
            <Layout>
                <EvaluateHelmet title="Assigned Entries"></EvaluateHelmet>
                <Container className="main">
                    <h1>Assigned Entries</h1>
                    {entryCards}
                </Container>
            </Layout>
        )
    }
}

export default AssignedEntries;
