import React, { Component } from 'react'
import EvaluateHelmet from "../../components/EvaluateHelmet";
import Layout from '../../components/Layout';
import { StateScreen } from '../../components/Screens';
import { Link } from "react-router-dom";
import Form from 'react-bootstrap/form'
import { Container } from '@material-ui/core';
import Slider from '@material-ui/core/Slider';
import { Card, CardActions, CardContent, CardHeader } from '@material-ui/core'
import Switch from '@material-ui/core/Switch';
import { SaveButton, SecondaryButton } from "../../components/Buttons";
import { get } from '../../Utils';

function Category(props) {
	//build main contents:
	let contents = null;
	if (!props.category.checked) {
		const marks = [
			{ value: 0, label: 0 },
			{
				value: props.category.maxScore,
				label: props.category.maxScore
			}
		];

		contents = (
			<CardContent>
				<Container maxWidth="md">
					<Slider
						defaultValue={props.value}
						aria-labelledby="discrete-slider"
						valueLabelDisplay="on"
						name={props.category.name}
						id={props.category.id}
						step={1}
						marks={marks}
						min={0}
						max={props.category.maxScore}
						onChangeCommitted={props.handleChange}
					/>
				</Container>
			</CardContent>);
	}

	let actions = null;
	if (props.category.subCategories.length !== 0) {
		actions = (
			<CardActions>
				<Switch 
					checked={props.category.checked} 
					id={props.category.id} 
					onChange={props.switchGradingMode} 
					color="primary"
				/>
				Grade Subcategories
			</CardActions>
		);
	}

	return (
		<div>
			<Card>
				<CardHeader
					title={props.category.name}
					action={<h2>{props.value}/{props.category.maxScore}</h2>}
				/>
				{contents}
				{actions}
			</Card>
			<br />
		</div>
	);
}

function SubCategory(props) {
	const marks = [];
	for (var i = 0; i <= props.subCat.maxScore; i++) {
		marks.push({
			value: i,
			label: i
		})
	}
	return (
		<div className="panel-body">
			<p><h5>{props.subCat.name}:</h5></p>
			<br/>
			<Container maxWidth="md">
				<Slider
					defaultValue={props.value}
					aria-labelledby="discrete-slider"
					valueLabelDisplay="on"
					name={props.subCat.name}
					id={props.subCat.id}
					step={1}
					marks={marks}
					min={0}
					max={props.subCat.maxScore}
					onChangeCommitted={props.handleChange}
				/>
			</Container>
			<br/>
		</div>
	);
}


class GradeEntry extends Component {

	emptyRubric = {
		categories: [],
		totalMaxScore: 0
	}

	emptyScoredRubric = {
		entry: null,
		template: null,
		judge: null,
		totalScore: 0,
		scoredCategories: [],
		judgeNote: ""
	}

	emptyScore = {
		template: 0,
		pointsAwarded: 0,
		subScores: [],
		checked: false
	}

	constructor(props) {
		super(props)
		let returnUrl;
		if (this.props.match.url.includes('chair')) {
			returnUrl = "/event/" + this.props.match.params.eventId + "/scorereport";
		} else {
			returnUrl = "/event/" + this.props.match.params.eventId + "/judge/" + this.props.match.params.judgeId + "/assignedentries";
		}

		this.state = {
			loading: 0,
			rubric: this.copy(this.emptyRubric),
			scoredRubric: this.copy(this.emptyScoredRubric),
			entry: {},
			judge: null,
			returnUrl: returnUrl
		}
		this.handleSubmit = this.handleSubmit.bind(this);
		this.handleChangeJudgeNote = this.handleChangeJudgeNote.bind(this);
		this.handleChangeCategory = this.handleChangeCategory.bind(this);
		this.switchGradingMode = this.switchGradingMode.bind(this);
	}

	async componentDidMount() {
		let {status, body} = await get(`${this.props.match.url}/getInfo`);
		if (status === 200) {//okay
			// get rubric:
			const rubric = body.rubric;
			const categories = rubric.categories;
			for (var i = 0; i < categories.length; i++) {
				categories[i].checked = false;
			}
			this.setState({ rubric: rubric, entry: body.entry, judge: body.judge });

			const sr = body.scoredRubric;
			if(sr != null){
				this.setState({scoredRubric: sr, loading: 1 });
			}
			else{
				//build scored rubric:
				const categories = this.state.rubric.categories;
				let scoredRubric = this.state.scoredRubric;
				scoredRubric.template = this.state.rubric;
				for (let i = 0; i < categories.length; i++) {
					let parentScore = this.copy(this.emptyScore);
					for (let j = 0; j < categories[i].subCategories.length; j++) {
						let subScore = this.copy(this.emptyScore);
						subScore.template = categories[i].subCategories[j];
						parentScore.subScores.push(subScore);
					}
					parentScore.template = categories[i];
					scoredRubric.scoredCategories.push(parentScore);
				}
				this.setState({ scoredRubric: scoredRubric, loading: 1 });
			}
		} else {
			this.setState({loading: status});
		}
	}

	copy(object) {
		return JSON.parse(JSON.stringify(object));
	}

	distributePointsAwarded(category, score) {
		let p = 0;
		let pa = score.pointsAwarded;
		let ms = category.maxScore
		for (let i = 0; i < score.subScores.length; i++) {
			let begin = this.eratosthenes(p, pa, ms);
			p += category.subCategories[i].maxScore;
			let end = this.eratosthenes(p, pa, ms);
			score.subScores[i].pointsAwarded = end - begin;
		}
	}

	eratosthenes(index, pointsAwarded, maxScore) {
		let value = index * pointsAwarded / maxScore;
		return Math.floor(value);
	}

	async handleChangeCategory(event, newValue, id) {
		//change category value:
		let scoredRubric = this.state.scoredRubric;
		scoredRubric.totalScore = 0;
		for (let i = 0; i < scoredRubric.scoredCategories.length; i++) {//iterate scores
			let score = scoredRubric.scoredCategories[i];
			if (score.template.id === id) {//if changing score
				score.pointsAwarded = newValue;
				this.distributePointsAwarded(score.template, score);
			} else {
				let changed = false;
				for (let k = 0; k < score.subScores.length; k++) {//iterate sub scores
					let subScore = score.subScores[k];
					if (subScore.template.id === id) {//if changing sub score
						subScore.pointsAwarded = newValue;
						changed = true;
						break;
					}
				}

				//recalculate score points awarded is sub score changed:
				if (changed) {
					score.pointsAwarded = 0;
					for (let k = 0; k < score.subScores.length; k++) {
						score.pointsAwarded += score.subScores[k].pointsAwarded;
					}
				}
			}

			scoredRubric.totalScore += score.pointsAwarded;
		}
		this.setState({scoredRubric: scoredRubric});
	}

	async handleChangeJudgeNote(event, newValue) {
		var scoredRubric = this.state.scoredRubric;
		scoredRubric.judgeNote = event.target.value;
		this.setState({scoredRubric: scoredRubric});
	}

	async handleSubmit(event) {
		//get rubric and scored rubric:
		let scoredRubric = this.state.scoredRubric;

		//post scored rubric:
		const response = await fetch(this.props.match.url + "/scoredrubric",
			{
				method: "PUT",
				headers: {
					"Accept": "application/json",
					"Content-Type": "application/json"
				},
				body: JSON.stringify(scoredRubric)
			}
		);

		//deal with response:
		if (response.status === 200) {
            this.props.history.push(this.state.returnUrl);
        }
        else if (response.status === 404){
            alert("Not found")
        }
        else if (response.status === 500){
            alert("Internal Server Error")
        }
	}

	async switchGradingMode(event) {
		const value = event.target.checked;	
		const id = event.target.id;
		var rubric = this.copy(this.state.rubric);
		for (var i = 0; i < rubric.categories.length; i++) {
			if (rubric.categories[i].id == id) {
				rubric.categories[i].checked = value;
			}
		}
		this.setState({ rubric: rubric });
	}

	render() {
		if ( this.state.loading !== 1 ) {
			return (
				<StateScreen state={this.state.loading}></StateScreen>
			);
		}

		//create categories list:
		var categoriesList = [];
		const categories = this.state.rubric.categories;
		const scores = this.state.scoredRubric.scoredCategories;
		for (let i = 0; i < categories.length; i++) {//iterate categories
			let catPoints = scores[i].pointsAwarded;
			let subCatList = [];
			if (categories[i].checked) {//if on sub category grading mode
				for (let j = 0; j < categories[i].subCategories.length; j++) {//iterate sub categories
					const sc = categories[i].subCategories[j];
					let subCatPoints = scores[i].subScores[j].pointsAwarded;
					let id = categories[i].subCategories[j].id;
					subCatList.push(<SubCategory subCat={sc} handleChange={(event, newValue) => this.handleChangeCategory(event, newValue, id)} value={subCatPoints} />);
				}
			}
			let id = categories[i].id;
			categoriesList.push(
				<Category category={categories[i]} value={catPoints} switchGradingMode={this.switchGradingMode} handleChange = {(event, newValue) => this.handleChangeCategory(event, newValue, id)}/>
			)
			categoriesList.push(subCatList);
		}

		return (
			<Layout>
				{/* Helmet is for the <head> */}
				<EvaluateHelmet title="Grade an Entry"></EvaluateHelmet>
				<Container className="main">
					<h1><em>{this.state.entry.title}</em></h1>
					{categoriesList}
					<Form>
						<Form.Label><h4>Judge Note</h4></Form.Label>
						<Form.Control as="textarea" rows="3" id="judgeNote" onChange={this.handleChangeJudgeNote} defaultValue={this.state.scoredRubric.judgeNote} />
					</Form>
					<br/>
					<Link to={this.state.returnUrl}>
						<SecondaryButton>Cancel</SecondaryButton>
					</Link>
					<SaveButton id="submit-btn" color='primary' onClick={() => this.handleSubmit()}>Save</SaveButton>
				</Container>
			</Layout>
		)
	}

}

export default GradeEntry;
