package com.evaluate.bootstrap;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Optional;
import java.util.Random;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.evaluate.models.*;
import com.evaluate.repositories.*;
import com.evaluate.services.HashService;

@Component
public class DevBootStrap implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private AccountRepository accountRepository;

	@Autowired
	private CategoryRepository categoryRepository;

	@Autowired
	private EntryRepository entryRepository;

	@Autowired
	private EventRepository eventRepository;

	@Autowired
	private RoleRepository roleRepository;

	@Autowired
	private RubricRepository rubricRepository;

	@Autowired
	private ScoreRepository scoreRepository;

	@Autowired
	private ScoredRubricRepository scoredRubricRepository;
	
	@Autowired
	private HashService hashService;

	@Override
	public void onApplicationEvent(ContextRefreshedEvent event) {
//		Optional<Account> opt = accountRepository.findByEmail("mscherger@fakemail.com");
//		if (opt.isEmpty()) {
//			List<String> emails = ccscAccounts();
//			Event ccsc = ccscEvent(emails);
//			ccscRubric(ccsc);
//			ccscEntries(ccsc);
//			ccscScoreReports(ccsc);
//		}
	}
	
	@Transactional
	private List<String> ccscAccounts() {
		List<String> emails = new ArrayList<String>();

		// ccsc chair:
		Account scherger = new Account();
		String email = "mscherger@fakemail.com";
		emails.add(email);
		scherger.setEmail(email);
		String ep = hashService.hash(email, "mspassword");
		scherger.setEncryptedPassword(ep);
		scherger.setFirstName("Michael");
		scherger.setLastName("Scherger");
		accountRepository.save(scherger);

		// ccsc judge 0:
		Account houston = new Account();
		email = "khouston@fakemail.com";
		emails.add(email);
		houston.setEmail(email);
		ep = hashService.hash(email, "khpassword");
		houston.setEncryptedPassword(ep);
		houston.setFirstName("Kenny");
		houston.setLastName("Houston");
		accountRepository.save(houston);

		// ccsc judge 1:
		Account dent = new Account();
		email = "jdent@fakemail.com";
		emails.add(email);
		dent.setEmail(email);
		ep = hashService.hash(email, "jdpassword");
		dent.setEncryptedPassword(ep);
		dent.setFirstName("Jordyn");
		dent.setLastName("Dent");
		accountRepository.save(dent);

		// ccsc judge 2:
		Account phan = new Account();
		email = "mphan@fakemail.com";
		emails.add(email);
		phan.setEmail(email);
		ep = hashService.hash(email, "mppassword");
		phan.setEncryptedPassword(ep);
		phan.setFirstName("Megan");
		phan.setLastName("Phan");
		accountRepository.save(phan);

		// ccsc judge 3:
		Account ha = new Account();
		email = "hha@fakemail.com";
		emails.add(email);
		ha.setEmail(email);
		ep = hashService.hash(email, "hhpassword");
		ha.setEncryptedPassword(ep);
		ha.setFirstName("Hau");
		ha.setLastName("Ha");
		accountRepository.save(ha);

		// ccsc judge 4:
		Account ye = new Account();
		email = "jye@fakemail.com";
		emails.add(email);
		ye.setEmail(email);
		ep = hashService.hash(email, "jypassword");
		ye.setEncryptedPassword(ep);
		ye.setFirstName("Jackie");
		ye.setLastName("Ye");
		accountRepository.save(ye);

		// ccsc judge 5:
		Account ghimire = new Account();
		email = "aghimire@fakemail.com";
		emails.add(email);
		ghimire.setEmail(email);
		ep = hashService.hash(email, "agpassword");
		ghimire.setEncryptedPassword(ep);
		ghimire.setFirstName("Action");
		ghimire.setLastName("Ghimire");
		accountRepository.save(ghimire);
		return emails;
	}

	@Transactional
	private Event ccscEvent(List<String> emails) {
		// init event:
		Event ccsc = new Event();
		ccsc.setName("CCSC South Central Region");
		ccsc.setDescription("A Computer Science Poster Competition");
		ccsc.setDateTime(new GregorianCalendar(2020, 4, 3, 11, 0));
		ccsc.refreshDateTime();
		ccsc.setLocation("University of Texas at Dallas");
		ccsc = eventRepository.save(ccsc);

		// add chair:
		Optional<Account> optScherger = accountRepository.findByEmail(emails.get(0));
		if (optScherger.isPresent()) {
			Account scherger = optScherger.get();
			Role ccscChair = new Chair();
			ccscChair.setAccount(scherger);
			ccscChair.setEvent(ccsc);
			ccscChair.setGroupNumber(0);
		}

		// add judges:
		for (int i = 1; i < emails.size(); i++) {
			Optional<Account> optJudge = accountRepository.findByEmail(emails.get(i));
			if (optJudge.isPresent()) {
				Account judge = optJudge.get();
				Role ccscJudge = new Judge();
				ccscJudge.setAccount(judge);
				ccscJudge.setEvent(ccsc);
				ccscJudge.setGroupNumber(i % 2);
			}
		}

		// save roles:
		roleRepository.save(ccsc.getChair());
		List<Judge> judges = ccsc.getJudges();
		for (Judge j : judges) {
			roleRepository.save(j);
		}

		return ccsc;
	}

	@Transactional
	private void ccscRubric(Event ccsc) {
		// set up rubric as a template:
		Rubric rubric = new Rubric();
		ccsc.setRubric(rubric);
		rubric = rubricRepository.save(rubric);
		ccsc = eventRepository.save(ccsc);

		Category cat = new Category("PRESENTATION");
		Category temp = new Category("Effectiveness", 5);
		temp.setDescription("Does it adequately explain the project?");
		cat.addSubCategory(temp);
		cat.addSubCategory(new Category("Clarity of Language", 5));
		cat.addSubCategory(new Category("Grammar", 5));
		cat.addSubCategory(new Category("Overall Visual Appeal and Clarity", 5));
		rubric.addCategory(cat);

		cat = new Category("ORGANIZATION OF POSTER");
		cat.addSubCategory(new Category("Purpose", 5));
		cat.addSubCategory(new Category("Methodology explained", 5));
		cat.addSubCategory(new Category("Emphasis given to important results", 5));
		temp = new Category("Significance", 5);
		temp.setDescription("Relation to other work");
		cat.addSubCategory(temp);
		rubric.addCategory(cat);

		cat = new Category("PROJECT DESIGN");
		cat.addSubCategory(new Category("Statement of objective", 5));
		cat.addSubCategory(new Category("Plan & execution", 10));
		cat.addSubCategory(new Category("Analysis of data", 10));
		cat.addSubCategory(new Category("Conclusion", 5));
		rubric.addCategory(cat);

		cat = new Category("FIGURES");
		cat.addSubCategory(new Category("Visibility/readability", 5));
		cat.addSubCategory(new Category("Accuracy", 5));
		cat.addSubCategory(new Category("Design", 5));
		rubric.addCategory(cat);

		cat = new Category("EXPERTISE");
		cat.addSubCategory(new Category("Ability to answer questions", 10));
		cat.addSubCategory(new Category("Knowledge of related literature/previous work", 5));
		rubric.addCategory(cat);

		for (int i = 0; i < rubric.getCategories().size(); i++) {
			categoryRepository.save(rubric.getCategories().get(i));
			for (int k = 0; k < rubric.getCategories().get(i).getSubCategories().size(); k++) {
				categoryRepository.save(rubric.getCategories().get(i).getSubCategories().get(k));
			}
		}
		rubric = rubricRepository.save(rubric);
	}

	@Transactional
	private void ccscEntries(Event ccsc) {
		// create entries:
		Entry entry = new Entry("Verification of Welfare Transactions on the Blockchain");
		entry.setDescription("We did a study on blockchain! It's cool.");
		entry.setTeamLeadName("Richard Feynman");
		entry.setTeamLeadEmail("rfeynman@fakemail.com");
		entry.setGroupNumer(0);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		entry = new Entry("Desirable Behaviors for Companion Bots in First-Person Shooters");
		entry.setDescription("This is a research project focusing on an aspect of game design.");
		entry.setTeamLeadName("Alan Turing");
		entry.setTeamLeadEmail("aturing@fakemail.com");
		entry.setGroupNumer(1);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		entry = new Entry("Creating a User-Friendly System to Facilitate Tracking and Entry of Internship Hours");
		entry.setDescription("We set out to make the best schedule tracker. Here are our results.");
		entry.setTeamLeadName("Grace Hopper");
		entry.setTeamLeadEmail("ghopper@fakemail.com");
		entry.setGroupNumer(0);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		entry = new Entry("Explainable Artificial Intelligence and Machine Learning: A reality rooted perspective");
		entry.setDescription(
				"We are used to the availability of big data generated in nearly all fields of science as a consequence "
						+ "of technological progress. However, the analysis of such data possess vast challenges. One of these r"
						+ "elates to the explainability of artificial intelligence (AI) or machine learning methods. Currently, "
						+ "many of such methods are non-transparent with respect to their working mechanism and for this reason "
						+ "are called black box models, most notably deep learning methods. However, it has been realized that t"
						+ "his constitutes severe problems for a number of fields including the health sciences and criminal jus"
						+ "tice and arguments have been brought forward in favor of an explainable AI. In this paper, we do not "
						+ "assume the usual perspective presenting explainable AI as it should be, but rather we provide a discu"
						+ "ssion what explainable AI can be. The difference is that we do not present wishful thinking but reali"
						+ "ty grounded properties in relation to a scientific theory beyond physics.");
		entry.setTeamLeadName("Dennis Ritchie");
		entry.setTeamLeadEmail("dritchie@fakemail.com");
		entry.setGroupNumer(1);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		entry = new Entry("Discrete graphical models -- an optimization perspective");
		entry.setDescription(
				"This monograph is about discrete energy minimization for discrete graphical models. It considers graphical mode"
						+ "ls, or, more precisely, maximum a posteriori inference for graphical models, purely as a combinatoria"
						+ "l optimization problem. Modeling, applications, probabilistic interpretations and many other aspects "
						+ "are either ignored here or find their place in examples and remarks only. It covers the integer linea"
						+ "r programming formulation of the problem as well as its linear programming, Lagrange and Lagrange dec"
						+ "omposition-based relaxations. In particular, it provides a detailed analysis of the polynomially solv"
						+ "able acyclic and submodular problems, along with the corresponding exact optimization methods. Major "
						+ "approximate methods, such as message passing and graph cut techniques are also described and analyzed"
						+ " comprehensively. The monograph can be useful for undergraduate and graduate students studying optimi"
						+ "zation or graphical models, as well as for experts in optimization who want to have a look into graph"
						+ "ical models. To make the monograph suitable for both categories of readers we explicitly separate the"
						+ " mathematical optimization background chapters from those specific to graphical models.");
		entry.setTeamLeadName("Kenneth Thompson");
		entry.setTeamLeadEmail("kthompson@fakemail.com");
		entry.setGroupNumer(0);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		entry = new Entry("A symmetric protocol to establish service level agreements");
		entry.setDescription(
				"We present a symmetrical protocol to repeatedly negotiate a desired service level between two part"
						+ "ies, where the service levels are taken from some totally ordered finite domain. The agr"
						+ "eed service level is selected from the levels proposed by both parties and parties can o"
						+ "nly decrease the desired service level during a negotiation. The correctness of the prot"
						+ "ocol is stated using modal formulas and its behaviour is explained using behavioural red"
						+ "uctions of the external behaviour modulo weak trace equivalence and divergence-preservin"
						+ "g branching bisimulation. Our protocol originates from an industrial use case and it tur"
						+ "ned out be remarkably tricky to design correctly.");
		entry.setTeamLeadName("Stephen Hawking");
		entry.setTeamLeadEmail("dritchie@fakemail.com");
		entry.setGroupNumer(1);
		ccsc.addEntry(entry);
		entryRepository.save(entry);

		eventRepository.save(ccsc);
	}

	@Transactional
	private void ccscScoreReports(Event ccsc) {
		// random number generator:
		Random rand = new Random((long) 256);

		// get rubric and entries:
		List<Entry> entries = ccsc.getEntries();
		Rubric rubric = ccsc.getRubric();

		// gather judge:
		List<Role> judges = new ArrayList<Role>();
		judges.add(ccsc.getChair());
		for (Judge j : ccsc.getJudges()) {
			judges.add(j);
		} // end judges loop

		for (Role j : judges) {
			for (Entry e : entries) {
				if (j.getGroupNumber() == e.getGroupNumber()) {//if matching group
					// grade the entry:
					ScoredRubric scoredRubric = new ScoredRubric(e, rubric);
					scoredRubric.setJudge(j);
					scoredRubricRepository.save(scoredRubric);
					for (Category c : rubric.getCategories()) {
						Score score = new Score(c);
						score.setScoredRubric(scoredRubric);
						scoreRepository.save(score);
						for (Category sc : c.getSubCategories()) {
							Score subScore = new Score(sc);

							// assign awarded points:
							int halfish = sc.getMaxScore() / 2 + 1;
							int points = rand.nextInt(sc.getMaxScore() - halfish);
							subScore.setPointsAwarded(points + halfish);

							subScore.setParentScore(score);
							scoreRepository.save(subScore);
						} // end sub categories loop
						scoreRepository.save(score);
					} // end categories loop
					scoredRubricRepository.save(scoredRubric);
				}
			} // end entries loop
		} // end judges loop
	}

}
