package com.evaluate.controllers;

import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.evaluate.models.Entry;
import com.evaluate.models.Role;
import com.evaluate.services.AssignGroupService;

@RestController
@RequestMapping("event/{eventId}/assigngroups")
public class AssignGroupController {

	@Autowired
	AssignGroupService assignGroupService;

	@GetMapping
	ResponseEntity<?> getInfo(@PathVariable Long eventId) {
		Map<String, Object> info = this.assignGroupService.getInfo(eventId);
		if (info != null) {
			return ResponseEntity.ok().body(info);
		} else {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
	}

	@DeleteMapping("/deleteentry/{entryId}")
	ResponseEntity<?> deleteEntry(@PathVariable long eventId, @PathVariable long entryId) {
		HttpStatus hs = assignGroupService.deleteEntry(eventId, entryId) ? HttpStatus.OK : HttpStatus.NOT_FOUND;
		return new ResponseEntity<>(hs);
	}

	@DeleteMapping("/deletejudge/{judgeId}")
	ResponseEntity<?> deleteJudge(@PathVariable Long eventId, @PathVariable Long judgeId) {
		return new ResponseEntity<>(assignGroupService.deleteJudge(eventId, judgeId));
	}

	@PostMapping
	ResponseEntity<?> saveEntries(@PathVariable Long eventId, @RequestBody SaveObjectWrapper saveObject) {
		List<Entry> entries = saveObject.getEntries();
		List<Role> judges = saveObject.getJudges();
		HttpStatus hs = assignGroupService.save(eventId, entries, judges) ? HttpStatus.OK : HttpStatus.NOT_FOUND;
		return new ResponseEntity<>(hs);
	}

}

class SaveObjectWrapper {
	private List<Entry> entries;
	private List<Role> judges;
	
	public List<Entry> getEntries() {
		return entries;
	}
	
	public void setEntires(List<Entry> entries) {
		this.entries = entries;
	}
	
	public List<Role> getJudges() {
		return judges;
	}
	
	public void setJudges(List<Role> judges) {
		this.judges = judges;
	}
	
}
