package com.evaluate.models;

import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

@Entity
@DiscriminatorValue("C")
public class Chair extends Role {

	private static final long serialVersionUID = 1L;

	public Chair() {
		super();
		groupNumber = -1;
	}

	public void setEvent(Event event) {
		// check if event already referenced
		if (this.event != null && this.event.equals(event)) {
			return;
		}

		Event oldEvent = this.event;// save old event
		this.event = event;
		if (oldEvent != null && this.equals(oldEvent.getChair())) {
			// remove reference to this:
			oldEvent.setChair(null);
		}

		// add new reference to this:
		if (event != null && !this.equals(event.getChair())) {
			// add reference:
			event.setChair(this);
		}
	}

	/*
	 * Sets group number. Must be in the range -1 <= groupNumber < 5. -1 represents
	 * no group. If the chair has no group they will not be involved in the judging
	 * process.
	 */
	public boolean setGroupNumber(int groupNumber) {
		// error check:
		if (groupNumber < -1 || groupNumber >= 5) {// if out of range
			return false;
		}

		// set new group number:
		this.groupNumber = groupNumber;
		return true;
	}

	@Override
	public String toString() {
		return "Chair [" + super.toString();
	}

}
