package com.evaluate.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.evaluate.models.Account;
import com.evaluate.models.Entry;
import com.evaluate.models.Event;
import com.evaluate.models.Role;
import com.evaluate.models.ScoredRubric;
import com.evaluate.repositories.EventRepository;
import com.evaluate.repositories.RoleRepository;

@Service
public class AssignedEntriesService {

	private ResponseEntity<?> notFoundRE = new ResponseEntity<>(HttpStatus.NOT_FOUND);

	@Autowired
	private RoleRepository roleRepository;
	@Autowired
	private EventRepository eventRepository;

	public ResponseEntity<?> getListAssignedEntries(long eventId, long judgeId) {
		// get relations:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Optional<Role> optJudge = roleRepository.findById(judgeId);
		if (optEvent.isEmpty() || optJudge.isEmpty()) {
			return notFoundRE;
		}
		Event event = optEvent.get();
		Role judge = optJudge.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		if (user.getId() != judge.getAccount().getId() || judge.getEvent().getId() != event.getId()) {
			return notFoundRE;
		}

		// get assigned entries:
		int groupNumber = judge.getGroupNumber();
		List<Entry> entriesList = event.getEntries();
		List<Entry> assignedList = new ArrayList<Entry>();
		// Get List of assigned entries with group number
		for (Entry entry : entriesList) {
			if (entry.getGroupNumber() == groupNumber) {
				assignedList.add(entry);
			}
		}

		// get scores for assigned entries:
		Set<ScoredRubric> scoredRubrics = judge.getScoredRubrics();
		List<Integer> scoreList = new ArrayList<Integer>();
		for (int i = 0; i < assignedList.size(); i++) {// iterate assigned entries
			Entry ae = assignedList.get(i);
			boolean added = false;
			for (ScoredRubric sr : scoredRubrics) {// iterate scored rubrics
				if (sr.getEntry().getId() == ae.getId()) {// if match
					// add to list:
					scoreList.add(sr.getTotalScore());
					added = true;
					break;
				}
			}
			if (!added) {// if no matching scored rubric found
				// assign score of zero:
				scoreList.add(0);
			}
		}

		// build response map:
		Map<String, Object> response = new HashMap<>();
		response.put("assignedEntries", assignedList);
		response.put("scores", scoreList);

		return ResponseEntity.ok(response);
	}

}
