package com.evaluate.services;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import com.evaluate.models.Account;
import com.evaluate.models.Event;
import com.evaluate.models.InviteCode;
import com.evaluate.models.Judge;
import com.evaluate.models.Role;
import com.evaluate.repositories.AccountRepository;
import com.evaluate.repositories.InviteCodeRepository;
import com.evaluate.repositories.RoleRepository;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

@Service
public class DashboardService {

	@Autowired
	InviteCodeRepository inviteCodeRepository;
	@Autowired
	RoleRepository roleRepository;
	@Autowired
	AccountRepository accountRepository;

	public Map<String, Object> getDashboard() {
		Map<String, Object> dashboard = new HashMap<>();
		Subject currentUser = SecurityUtils.getSubject();

		Account account = (Account) currentUser.getPrincipal();
		account = accountRepository.findById(account.getId()).get();
		dashboard.put("accountInfo", Optional.of(account));
		List<Role> roleList = account.getRoles();
		List<Event> joinedEvents = new ArrayList<>();
		List<Event> createdEvents = new ArrayList<>();
		for (Role role : roleList) {
			if (role.getClass().getName().contains("Chair")) {
				createdEvents.add(role.getEvent());
			} else {
				joinedEvents.add(role.getEvent());
			}
		}
		dashboard.put("joinedEvents", joinedEvents);
		dashboard.put("createdEvents", createdEvents);
		return dashboard;
	}

	public ResponseEntity<?> joinEvent(String code) throws URISyntaxException {
		// get invite code:
		Optional<InviteCode> optInviteCode = inviteCodeRepository.findById(code);
		if (optInviteCode.isEmpty()) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		InviteCode inviteCode = optInviteCode.get();

		// get event:
		Event event = inviteCode.getEvent();

		// delete invite code:
		inviteCodeRepository.delete(inviteCode);

		// check that isn't already a judge:
		Account account = (Account) SecurityUtils.getSubject().getPrincipal();
		List<Role> roles = account.getRoles();
		for (Role r : roles) {
			if (r.getEvent().getId() == event.getId()) {
				return new ResponseEntity<>(HttpStatus.CONFLICT);
			}
		}

		// create new role as judge:
		Judge judge = new Judge();
		judge.setAccount(account);
		event.addJudge(judge);
		roleRepository.save(judge);

		return ResponseEntity.created(new URI("/event/" + event.getId())).body(event);
	}

}
