package com.evaluate.services;

import com.evaluate.models.Entry;
import com.evaluate.models.Event;
import com.evaluate.repositories.EntryRepository;
import com.evaluate.repositories.EventRepository;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

@Service
public class EventRegisterService {

	@Autowired
	private EntryRepository entryRepository;

	@Autowired
	private EventRepository eventRepository;

	@Autowired
	private MailService mailService;

	/**
	 * After the participant submitted their entry, a new entry is created and
	 * stored in the database.
	 * 
	 * @param entry
	 * @return saved a new entry
	 */
	public ResponseEntity<Entry> createEntry(long eventId, Entry entry) throws URISyntaxException {
		// save the entry:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Event event = optEvent.get();
		entry.setEvent(event);
		Entry newEntry = entryRepository.save(entry);
		
		// email the team lead:
		String header = "Entry Registration for " + event.getName();
		String body = newEntry.getTeamLeadName() + ",\n\n";
		body += "Your submission to " + event.getName() + " has been received.\n\n";
		
		body += "Team Name: " + newEntry.getTeamName() + "\n";
		body += "Title: " + newEntry.getTitle() + "\n";
		body += "Description: " + newEntry.getDescription() + "\n";
		body += "Team Lead: " + newEntry.getTeamLeadName() + "\n";
		body += "Other Team Members: " + newEntry.getTeamMemberNames() + "\n\n";
		body += "If the details of your entry change, please contact the organizer of your event.";
		mailService.sendEmail(newEntry.getTeamLeadEmail(), header, body);
		
		return ResponseEntity.created(new URI("/event/" + eventId + "/entry/" + newEntry.getId())).body(newEntry);
	}

	public ResponseEntity<Event> getEvent(long eventId) {
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		Event event = optEvent.get();

		// remove sensitive information:
		event.setRubric(null);

		return ResponseEntity.ok(event);
	}

}
