package com.evaluate.services;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.evaluate.models.*;
import com.evaluate.repositories.*;

@Service
public class GradeEntryChairService {

	@Autowired
	private ScoreRepository scoreRepository;
	@Autowired
	private ScoredRubricRepository scoredRurbicRepository;
	@Autowired
	private CategoryRepository categoryRepository;
	@Autowired
	private EventRepository eventRepository;
	@Autowired
	private EntryRepository entryRepository;
	@Autowired
	private RoleRepository roleRepository;

	public boolean saveOrUpdateScoredRubric(ScoredRubric scoredRubric, long eventId, long judgeId, long entryId) {
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Optional<Role> optJudge = roleRepository.findById(judgeId);
		Optional<Entry> optEntry = entryRepository.findById(entryId);
		if (optEvent.isEmpty() || optJudge.isEmpty() || optEntry.isEmpty()) {
			return false;
		}

		// set relations:
		Event event = optEvent.get();
		Rubric rubric = event.getRubric();
		Role judge = optJudge.get();
		Entry entry = optEntry.get();
		scoredRubric.setTemplate(rubric);
		scoredRubric.setJudge(judge);
		scoredRubric.setEntry(entry);

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		if (user.getId() != event.getChair().getAccount().getId()// if not chair
				|| judge.getEvent().getId() != event.getId()// if judge not for event
				|| entry.getEvent().getId() != event.getId()// if entry not for event
				|| entry.getGroupNumber() != judge.getGroupNumber()) {// if wrong group number
			return false;
		}

		Optional<ScoredRubric> optSr = scoredRurbicRepository.findByTemplateAndEntryAndJudge(rubric, entry, judge);

		// save scored rubric
		if (optSr.isPresent()) {
			scoredRubric.setId(optSr.get().getId());
		}

		scoredRurbicRepository.save(scoredRubric);

		// save scored categories:
		for (Score sc : scoredRubric.getScoredCategories()) {
			saveOrUpdateScore(sc);
		}
		return true;
	}

	public void saveOrUpdateScore(Score score) {
		// set template:
		Category category = this.categoryRepository.findById(score.getTemplate().getId()).get();
		score.setTemplate(category);

		if (score.getParentScore() == null && score.getScoredRubric() != null) {
			ScoredRubric scoredrubric = this.scoredRurbicRepository.findById(score.getScoredRubric().getId()).get();
			Optional<Score> opts = scoreRepository.findByScoredRubricAndTemplate(scoredrubric, category);
			if (!opts.isEmpty()) {
				score.setId(opts.get().getId());
			}
		} else if (score.getParentScore() != null && score.getScoredRubric() == null) {
			Score parentScore = this.scoreRepository.findById(score.getParentScore().getId()).get();

			Optional<Score> opts = scoreRepository.findByParentScoreAndTemplate(parentScore, category);
			if (!opts.isEmpty()) {
				score.setId(opts.get().getId());
			}
		}

		scoreRepository.save(score);

		// save sub scores:
		for (Score ss : score.getSubScores()) {
			saveOrUpdateScore(ss);
		}
	}

	public Map<String, Object> getInfo(long eventId, long judgeId, long entryId) {
		Optional<Role> optJudge = roleRepository.findById(judgeId);
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Optional<Entry> optEntry = entryRepository.findById(entryId);
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();

		// get data:
		if (optJudge.isEmpty() || optEvent.isEmpty() || optEntry.isEmpty()) {
			return null;
		}
		Event event = optEvent.get();
		Role judge = optJudge.get();
		Rubric rubric = event.getRubric();
		Entry entry = optEntry.get();

		// check authorization:
		if (user.getId() != event.getChair().getAccount().getId()// if not chair
				|| judge.getEvent().getId() != event.getId()// if judge not for event
				|| entry.getEvent().getId() != event.getId()// if entry not for event
				|| entry.getGroupNumber() != judge.getGroupNumber()) {// if wrong group number
			return null;
		}

		// fetch Scored Rubric:
		Optional<ScoredRubric> optScoredRubric = scoredRurbicRepository.findByTemplateAndEntryAndJudge(rubric, entry,
				judge);

		// Put into Hashmap
		Map<String, Object> fetch = new HashMap<>();
		fetch.put("rubric", rubric);
		fetch.put("judge", judge);
		fetch.put("entry", entry);
		if (optScoredRubric.isPresent()) {
			fetch.put("scoredRubric", optScoredRubric.get());
		}

		return fetch;
	}
}
