package com.evaluate.services;

import java.util.*;
import com.evaluate.models.*;
import com.evaluate.repositories.*;

import org.apache.shiro.SecurityUtils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ScoreReportService {

    @Autowired
    private EventRepository eventRepository;

    private Map<Integer, Map<String, Set<ScoredRubric>>> organizeByGroup(Event event){
        Map<Integer, Map<String, Set<ScoredRubric>>> scoredEntries = new HashMap<>();
        List<Entry> entryList = event.getEntries();
        for(Entry entry : entryList){
            Integer groupNum = entry.getGroupNumber();
            Map<String, Set<ScoredRubric>> groupByEntry = new HashMap<>();
            Set<ScoredRubric> scoredRubricSet = entry.getScoredRubrics();
            groupByEntry.put(entry.getTitle(), scoredRubricSet);
           if(!scoredEntries.containsKey(groupNum)){
                scoredEntries.put(groupNum, groupByEntry);
           }else{
                scoredEntries.get(groupNum).put(entry.getTitle(), scoredRubricSet);
           }
        }
        return scoredEntries;
    }
    /**
     * 
     * @param eventId
     * @return a HashMap that contains scored rubrics and event statistics such as
     *         list of judges and their accounts to get their names
     **/
    public Map<String, Object> getEventStat(long eventId) {
        Account user = (Account) SecurityUtils.getSubject().getPrincipal();
        Optional<Event> optEvent = eventRepository.findById(eventId);
        if (optEvent.isEmpty()) {
        	return null;
        }
        Event event = optEvent.get();
        if(user.getId() != event.getChair().getAccount().getId()){
            return null;
        }
        List<Category> catList = new ArrayList<Category>();
        if(event.getRubric() != null){
            catList = event.getRubric().getCategories();
        }
        Map<String, Object> eventStats = new HashMap<>();
        eventStats.put("catList", catList);
        eventStats.put("eventInfo", event);
        List<Judge> listOfJudges = new ArrayList<Judge>();
        if(!event.getJudges().isEmpty()){
            listOfJudges = event.getJudges();
        }
        List<Account> judgeAccounts = new ArrayList<Account>();
        for (Judge judge : listOfJudges) {
            Account jAccount = judge.getAccount();
            judgeAccounts.add(jAccount);
        }
        if(event.getChair().getGroupNumber() >= 0 && event.getChair().getGroupNumber() <= 4){
            judgeAccounts.add(event.getChair().getAccount());
        }
        eventStats.put("judges", judgeAccounts);
        eventStats.put("scoredEntries", organizeByGroup(event));
        return eventStats;
    }
}