package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class AccountTest {

	private Account acc;
	
	private Chair chair;
	private Judge judge;
	private List<Role> roles;
	
	@BeforeEach
	void setUp() throws Exception {
		acc = new Account();
		
		chair = new Chair();
		judge = new Judge();
		roles = new ArrayList<Role>();
		for (int i = 0; i < 4; i++) {
			roles.add(new Judge());
		}
		roles.add(new Chair());
	}
	
	@Test
	void testAccount() {
		acc = new Account();
		assertNull(acc.getEmail());
		assertNull(acc.getEncryptedPassword());
		assertNull(acc.getFirstName());
		assertNull(acc.getLastName());
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		assertEquals("", acc.getEncryptedResetCode());
	}

	@Test
	void testSetEmail() {
		//verify null at start
		assertNull(acc.getEmail());
		
		//set email
		String email = "anEmail@mail.generic";
		acc.setEmail(email);
		assertEquals(email, acc.getEmail());
		
		//set null
		acc.setEmail(null);
		assertNull(acc.getEmail());
	}

	@Test
	void testSetEncryptedPassword() {
		//verify null at start
		assertNull(acc.getEncryptedPassword());
		
		//set email
		String pw = "asdf1234";
		acc.setEncryptedPassword(pw);
		assertEquals(pw, acc.getEncryptedPassword());
		
		//set null
		acc.setEncryptedPassword(null);
		assertNull(acc.getEncryptedPassword());
	}

	@Test
	void testSetFirstName() {
		//verify null at start
		assertNull(acc.getFirstName());
		
		//set email
		String fn = "John";
		acc.setFirstName(fn);
		assertEquals(fn, acc.getFirstName());
		
		//set null
		acc.setFirstName(null);
		assertNull(acc.getFirstName());
	}

	@Test
	void testSetLastName() {
		//verify null at start
		assertNull(acc.getLastName());
		
		//set email
		String ln = "Doe";
		acc.setLastName(ln);
		assertEquals(ln, acc.getLastName());
		
		//set null
		acc.setLastName(null);
		assertNull(acc.getLastName());
	}

	@SuppressWarnings("unchecked")
	@Test
	void testSetRoles() {
		//verify empty at first
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		
		//set roles
		List<Role> rCopy = (List<Role>)((ArrayList<Role>)roles).clone();
		acc.setRoles(rCopy);
		assertIterableEquals(roles, acc.getRoles());
		for (Role r : roles) {
			assertEquals(acc, r.getAccount());
		}
		
		//set Same roles
		List<Role> rCopy2 = (List<Role>)((ArrayList<Role>)roles).clone();
		acc.setRoles(rCopy2);
		assertIterableEquals(roles, acc.getRoles());
		for (Role r : roles) {
			assertEquals(acc, r.getAccount());
		}
		
		//prep different roles
		List<Role> diffRoles = new ArrayList<Role>();
		for (int i = 0; i < 3; i++) {
			diffRoles.add(new Chair());
		}
		diffRoles.add(new Judge());
		
		//set different roles
		List<Role> drCopy = (List<Role>)((ArrayList<Role>)diffRoles).clone();
		acc.setRoles(drCopy);
		assertIterableEquals(diffRoles, acc.getRoles());
		for (Role dr : diffRoles) {
			assertEquals(acc, dr.getAccount());
		}
		for (Role r : roles) {
			assertNull(r.getAccount());
		}
		
		//set null
		acc.setRoles(null);
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		for (Role dr : diffRoles) {
			assertNull(dr.getAccount());
		}
	}

	@Test
	void testAddRole() {
		//verify empty at start
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		
		//add role
		assertTrue(acc.addRole(judge));
		assertTrue(acc.getRoles().contains(judge));
		assertEquals(acc, judge.getAccount());
		
		//try to add SAME role
		assertFalse(acc.addRole(judge));
		assertTrue(acc.getRoles().contains(judge));
		assertEquals(acc, judge.getAccount());
		assertEquals(1, acc.getRoles().size());
		
		//add another role
		assertTrue(acc.addRole(chair));
		assertTrue(acc.getRoles().contains(chair));
		assertEquals(acc, chair.getAccount());
		
		//try add null
		assertFalse(acc.addRole(null));
		assertFalse(acc.getRoles().contains(null));
		assertEquals(2, acc.getRoles().size());
	}

	@SuppressWarnings("unchecked")
	@Test
	void testRemoveRoleRole() {
		//verify empty at first
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		
		//set list
		List<Role> rCopy = (List<Role>)((ArrayList<Role>)roles).clone();
		acc.setRoles(rCopy);
		assertEquals(roles.size(), acc.getRoles().size());
		
		//try to remove judge not in list
		assertFalse(acc.removeRole(new Judge()));
		assertEquals(roles.size(), acc.getRoles().size());
		
		//try to remove null
		assertFalse(acc.removeRole(null));
		assertEquals(roles.size(), acc.getRoles().size());
		
		//remove one by one
		for ( Role r : roles ) {
			assertTrue(acc.removeRole(r));
		}
		assertEquals(0, acc.getRoles().size());
	}

	@SuppressWarnings("unchecked")
	@Test
	void testRemoveRoleInt() {//TODO check if valid
		//verify empty at first
		assertEquals(new ArrayList<Role>(), acc.getRoles());
		
		//set list
		List<Role> jCopy = (List<Role>)((ArrayList<Role>)roles).clone();
		acc.setRoles(jCopy);
		assertEquals(roles.size(), acc.getRoles().size());
		
		//try to remove role out of bounds
		try {
			assertNull(acc.removeRole(-1));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index -1 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(roles.size(), acc.getRoles().size());
		
		try {
			assertNull(acc.removeRole(5));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index 5 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(roles.size(), acc.getRoles().size());
		
		//remove one by one
		for ( int i = roles.size()-1; i >= 0; i--) {
			assertEquals(roles.get(i), acc.removeRole(i));
		}
		assertEquals(0, acc.getRoles().size());
	}

}
