package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.HashSet;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class EntryTest {
	
	private Entry entry;
	private Event event;
	private ScoredRubric scoredRubric;
	private HashSet<ScoredRubric> scoredRubrics;

	@BeforeEach
	void setUp() throws Exception {
		entry = new Entry();
		event = new Event();

		scoredRubric = new ScoredRubric();
		scoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 4; i++) {
			scoredRubrics.add(new ScoredRubric());
		}
	}

	@Test
	void testEntry() {
		entry = new Entry();
		assertEquals("", entry.getTitle());
		assertNull(entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
	}

	@Test
	void testEntryString() {
		//normal args
		String title = "Test Title";
		entry = new Entry(title);
		assertEquals(title, entry.getTitle());
		assertNull(entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
		
		//bad args
		entry = new Entry(null);
		assertEquals("", entry.getTitle());
		assertNull(entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
	}

	@Test
	void testEntryStringString() {
		//normal args
		String title = "Test Title";
		String desc = "A normal description.";
		entry = new Entry(title, desc);
		assertEquals(title, entry.getTitle());
		assertEquals(desc, entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
		
		//bad title arg
		entry = new Entry(null, desc);
		assertEquals("", entry.getTitle());
		assertEquals(desc, entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
		
		//bad description arg
		entry = new Entry(title, null);
		assertEquals(title, entry.getTitle());
		assertNull(entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
		
		//all bad args
		entry = new Entry(null, null);
		assertEquals("", entry.getTitle());
		assertNull(entry.getDescription());
		assertNull(entry.getEvent());
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		assertNull(entry.getTeamName());
		assertNull(entry.getTeamLeadName());
		assertNull(entry.getTeamLeadEmail());
		assertNull(entry.getTeamMemberNames());
		assertEquals(0, entry.getGroupNumber());
	}

	@Test
	void testSetTitle() {
		//verify empty title at start:
		assertEquals("", entry.getTitle());
		
		//set title
		entry.setTitle("Test Title");
		assertEquals("Test Title", entry.getTitle());
		
		//change title:
		entry.setTitle("A new title");
		assertEquals("A new title", entry.getTitle());
		
		//try setting title to null:
		entry.setTitle(null);
		
		//verify equal to empty string instead:
		assertNotNull(entry.getTitle());
		assertEquals("", entry.getTitle());
	}

	@Test
	void testSetDescription() {
		//verify null description at start:
		assertNull(entry.getDescription());
		
		//set description:
		entry.setDescription("Here's a description");
		assertEquals("Here's a description", entry.getDescription());
		
		//change description:
		entry.setDescription("Another description");
		assertEquals("Another description", entry.getDescription());
		
		//set description to null:
		entry.setDescription(null);
		assertNull(entry.getDescription());
	}

	@Test
	void testSetEvent() {
		//verify null at first
		assertNull(entry.getEvent());
		
		//set event
		entry.setEvent(event);
		assertEquals(event, entry.getEvent());
		assertTrue(event.getEntries().contains(entry));
		
		//set same event
		entry.setEvent(event);
		assertEquals(event, entry.getEvent());
		assertTrue(event.getEntries().contains(entry));
		assertEquals(1, event.getEntries().size());
		
		//set different event
		Event event2 = new Event();
		entry.setEvent(event2);
		assertEquals(event2, entry.getEvent());
		assertTrue(event2.getEntries().contains(entry));
		assertFalse(event.getEntries().contains(entry));
		
		//set null
		entry.setEvent(null);
		assertNull(entry.getEvent());
		assertFalse(event2.getEntries().contains(entry));
	}
	
	@Test
	void testSetScoredRubrics() {
		//verify empty scored rubric set at first:
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		
		//set scored rubrics:
		@SuppressWarnings("unchecked")
		HashSet<ScoredRubric> srCopy = (HashSet<ScoredRubric>)scoredRubrics.clone();
		entry.setScoredRubrics(srCopy);
		
		//verify relations:
		assertEquals(scoredRubrics, entry.getScoredRubrics());
		for (ScoredRubric sr : scoredRubrics) {
			assertEquals(entry, sr.getEntry());
		}
		
		//set SAME scored rubrics:
		@SuppressWarnings("unchecked")
		HashSet<ScoredRubric> srCopy2 = (HashSet<ScoredRubric>)scoredRubrics.clone();
		entry.setScoredRubrics(srCopy2);
		
		//verify relations still exists:
		assertEquals(scoredRubrics, entry.getScoredRubrics());
		for (ScoredRubric sr : scoredRubrics) {
			assertEquals(entry, sr.getEntry());
		}
		
		//set different scored rubrics:
		HashSet<ScoredRubric> newScoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 3; i++) {
			newScoredRubrics.add(new ScoredRubric());
		}
		@SuppressWarnings("unchecked")
		HashSet<ScoredRubric> srCopy3 = (HashSet<ScoredRubric>)newScoredRubrics.clone();
		entry.setScoredRubrics(srCopy3);
		
		//verify relations set:
		assertEquals(newScoredRubrics, entry.getScoredRubrics());
		for (ScoredRubric sr : newScoredRubrics) {
			assertEquals(entry, sr.getEntry());
		}
		
		//verify old relations removed:
		for (ScoredRubric sr : scoredRubrics) {
			assertNull(sr.getEntry());
			assertFalse(entry.getScoredRubrics().contains(sr));
		}
		
		//set null:
		entry.setScoredRubrics(null);
		
		//verify not null:
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		
		//verify relations gone:
		for (ScoredRubric sr : newScoredRubrics) {
			assertNull(sr.getEntry());
		}
	}

	@Test
	void testAddScoredRubric() {
		//verify empty set at first:
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
		
		//add new scored rubric:
		entry.addScoredRubric(scoredRubric);
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));
		assertEquals(entry, scoredRubric.getEntry());
		
		//try adding SAME scored rubric:
		entry.addScoredRubric(scoredRubric);
		
		//verify size did not change:
		assertEquals(1, entry.getScoredRubrics().size());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));
		assertEquals(entry, scoredRubric.getEntry());
		
		ScoredRubric newSR = new ScoredRubric();
		entry.addScoredRubric(newSR);
		assertEquals(2, entry.getScoredRubrics().size());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));
		assertEquals(entry, scoredRubric.getEntry());
		
		//set up entry with previous set of scored rubric:
		entry.setScoredRubrics(scoredRubrics);
		int srSize = entry.getScoredRubrics().size();
		
		//add another scored rubric:
		entry.addScoredRubric(scoredRubric);
		assertEquals(srSize+1, entry.getScoredRubrics().size());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));
		assertEquals(entry, scoredRubric.getEntry());
	}

	@Test
	void testRemoveScoredRubric() {
		//give entry scored rubrics:
		entry.setScoredRubrics(scoredRubrics);
		assertNotEquals(0, entry.getScoredRubrics().size());
		
		//remove one by one:
		for (ScoredRubric sr : scoredRubrics) {
			entry.removeScoredRubric(sr);
			assertNull(sr.getEntry());
		}
		assertEquals(new HashSet<ScoredRubric>(), entry.getScoredRubrics());
	}
	
	@Test
	void testSetGroupNumber() {
		//verify zero at start
		assertEquals(0, entry.getGroupNumber());

		//set in range
		assertTrue(entry.setGroupNumer(4));
		assertEquals(4, entry.getGroupNumber());
		
		//set out of range
		assertFalse(entry.setGroupNumer(5));
		assertEquals(4, entry.getGroupNumber());
		
		//set out of range
		assertFalse(entry.setGroupNumer(-1));
		assertEquals(4, entry.getGroupNumber());
	}
	
}
