package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class EventTest {
	
	private Event event;
	private Chair chair;
	private Rubric rubric;
	
	private Judge judge;
	private List<Judge> judges;
	
	private Entry entry;
	private List<Entry> entries;
	
	private InviteCode inviteCode;
	private List<InviteCode> inviteCodes;
	
	@BeforeEach
	void setUp() throws Exception {
		event = new Event();
		chair = new Chair();
		rubric = new Rubric();
		
		judge = new Judge();
		judges = new ArrayList<Judge>();
		for (int i = 0; i < 4; i++) {
			judges.add(new Judge());
		}
		
		entry = new Entry("An Entry");
		entries = new ArrayList<Entry>();
		for (int i = 0; i < 5; i++) {
			entries.add(new Entry("Entry " + i));
		}
		
		inviteCode = new InviteCode();
		inviteCodes = new ArrayList<InviteCode>();
		for (int i = 0; i < 5; i++) {
			inviteCodes.add(new InviteCode());
		}
	}

	@Test
	void testEvent() {
		event = new Event();
		assertEquals("", event.getName());
		assertNull(event.getDescription());
		assertNull(event.getDateTime());
		assertNull(event.getLocation());
		assertNull(event.getChair());
		assertEquals(0, event.getJudges().size());
		assertNull(event.getRubric());
		assertEquals(0, event.getEntries().size());
		assertEquals(0, event.getInviteCodes().size());
		assertEquals(0, event.getInviteCounter());
	}
	
	@Test
	void testSetName() {
		//verify empty at first
		assertEquals("", event.getName());
		
		//set a name
		String name1 = "Name 1";
		event.setName(name1);
		assertEquals(name1, event.getName());
		
		//set null
		event.setName(null);
		assertEquals("", event.getName());
		
		//set empty
		event.setName("");
		assertEquals("", event.getName());
	}
	
	@Test
	void testSetDescription() {
		//verify null at first
		assertNull(event.getDescription());
		
		//set a description
		String desc1 = "Description 1";
		event.setDescription(desc1);
		assertEquals(desc1, event.getDescription());
		
		//set empty string
		event.setDescription("");
		assertNull(event.getDescription());
		
		//set another description
		String desc2 = "Another description.";
		event.setDescription(desc2);
		assertEquals(desc2, event.getDescription());
		
		//set null
		event.setDescription(null);
		assertNull(event.getDescription());
	}
	
	@Test
	void testSetChair() {
		//verify null at first
		assertNull(event.getChair());
		
		//set chair
		event.setChair(chair);
		assertEquals(chair, event.getChair());
		assertEquals(event, chair.getEvent());
		
		//set SAME chair
		event.setChair(chair);
		assertEquals(chair, event.getChair());
		assertEquals(event, chair.getEvent());
		
		//set different chair
		Chair chair2 = new Chair();
		event.setChair(chair2);
		assertEquals(chair2, event.getChair());
		assertEquals(event, chair2.getEvent());
		assertNull(chair.getEvent());
		
		//set null
		event.setChair(null);
		assertNull(event.getChair());
		assertNull(chair2.getEvent());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testSetJudges() {
		//verify empty at first
		assertEquals(new ArrayList<Judge>(), event.getJudges());
		
		//set list
		List<Judge> jCopy = (List<Judge>)((ArrayList<Judge>)judges).clone();
		event.setJudges(jCopy);
		assertIterableEquals(judges, event.getJudges());
		for (Judge j : judges) {
			assertEquals(event, j.getEvent());
		}
		
		//set Same list:
		List<Judge> jCopy2 = (List<Judge>)((ArrayList<Judge>)judges).clone();
		event.setJudges(jCopy2);
		assertIterableEquals(judges, event.getJudges());
		for (Judge j : judges) {
			assertEquals(event, j.getEvent());
		}
		
		//prep different list
		List<Judge> diffJudges = new ArrayList<Judge>();
		for (int i = 0; i < 3; i++) {
			diffJudges.add(new Judge());
		}
		
		//set different list
		List<Judge> djCopy = (List<Judge>)((ArrayList<Judge>)diffJudges).clone();
		event.setJudges(djCopy);
		assertIterableEquals(diffJudges, event.getJudges());
		for (Judge dj : diffJudges) {
			assertEquals(event, dj.getEvent());
		}
		for (Judge j : judges) {
			assertNull(j.getEvent());
		}
		
		//set null
		event.setJudges(null);
		assertEquals(new ArrayList<Judge>(), event.getJudges());
		for (Judge dj : diffJudges) {
			assertNull(dj.getEvent());
		}
	}

	@Test
	void testAddJudge() {
		//verify empty at first
		assertEquals(new ArrayList<Judge>(), event.getJudges());
		
		//add judge
		assertTrue(event.addJudge(judge));
		assertTrue(event.getJudges().contains(judge));
		assertEquals(event, judge.getEvent());
		
		//add another judge
		Judge judge2 = new Judge();
		assertTrue(event.addJudge(judge2));
		assertTrue(event.getJudges().contains(judge));
		assertTrue(event.getJudges().contains(judge2));
		assertEquals(event, judge.getEvent());
		assertEquals(event, judge2.getEvent());
		
		//try add null
		assertFalse(event.addJudge(null));
		assertFalse(event.getJudges().contains(null));
		assertEquals(2, event.getJudges().size());
		
		//try add duplicate
		assertFalse(event.addJudge(judge));
		assertTrue(event.getJudges().contains(judge));
		assertTrue(event.getJudges().contains(judge2));
		assertEquals(event, judge.getEvent());
		assertEquals(event, judge2.getEvent());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveJudgeJudge() {
		//verify empty at first
		assertEquals(new ArrayList<Judge>(), event.getJudges());
		
		//set list
		List<Judge> jCopy = (List<Judge>)((ArrayList<Judge>)judges).clone();
		event.setJudges(jCopy);
		assertEquals(judges.size(), event.getJudges().size());
		
		//try to remove judge not in list
		assertFalse(event.removeJudge(new Judge()));
		assertEquals(judges.size(), event.getJudges().size());
		
		//try to remove null
		assertFalse(event.removeJudge(null));
		assertEquals(judges.size(), event.getJudges().size());
		
		//remove one by one
		for ( Judge j : judges ) {
			assertTrue(event.removeJudge(j));
		}
		assertEquals(0, event.getJudges().size());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveJudgeInt() {
		//verify empty at first
		assertEquals(new ArrayList<Judge>(), event.getJudges());
		
		//set list
		List<Judge> jCopy = (List<Judge>)((ArrayList<Judge>)judges).clone();
		event.setJudges(jCopy);
		assertEquals(judges.size(), event.getJudges().size());
		
		//try to remove judge out of bounds
		try {
			assertNull(event.removeJudge(-1));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index -1 out of bounds for length 4";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(judges.size(), event.getJudges().size());
		
		try {
			assertNull(event.removeJudge(4));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index 4 out of bounds for length 4";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(judges.size(), event.getJudges().size());
		
		//remove one by one
		for ( int i = judges.size()-1; i >= 0; i--) {
			assertEquals(judges.get(i), event.removeJudge(i));
		}
		assertEquals(0, event.getJudges().size());
	}
	
	@Test
	void testSetRubric() {
		//verify null at first
		assertNull(event.getRubric());
		
		//set rubric
		event.setRubric(rubric);
		assertEquals(rubric, event.getRubric());
		assertEquals(event, rubric.getEvent());
		
		//set same rubric
		event.setRubric(rubric);
		assertEquals(rubric, event.getRubric());
		assertEquals(event, rubric.getEvent());
		
		//set different rubric
		Rubric rubric2 = new Rubric();
		event.setRubric(rubric2);
		assertEquals(rubric2, event.getRubric());
		assertEquals(event, rubric2.getEvent());
		assertNull(rubric.getEvent());
		
		//set null
		event.setRubric(null);
		assertNull(event.getRubric());
		assertNull(rubric2.getEvent());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testSetEntries() {
		//verify empty at first
		assertEquals(new ArrayList<Entry>(), event.getEntries());
		
		//set list
		List<Entry> eCopy = (List<Entry>)((ArrayList<Entry>)entries).clone();
		event.setEntries(eCopy);
		assertIterableEquals(entries, event.getEntries());
		for (Entry e : entries) {
			assertEquals(event, e.getEvent());
		}
		
		//set Same list:
		List<Entry> eCopy2 = (List<Entry>)((ArrayList<Entry>)entries).clone();
		event.setEntries(eCopy2);
		assertIterableEquals(entries, event.getEntries());
		for (Entry e : entries) {
			assertEquals(event, e.getEvent());
		}
		
		//prep different list
		List<Entry> diffEntries = new ArrayList<Entry>();
		for (int i = 0; i < 3; i++) {
			diffEntries.add(new Entry());
		}
		
		//set different list
		List<Entry> deCopy = (List<Entry>)((ArrayList<Entry>)diffEntries).clone();
		event.setEntries(deCopy);
		assertIterableEquals(diffEntries, event.getEntries());
		for (Entry de : diffEntries) {
			assertEquals(event, de.getEvent());
		}
		for (Entry e : entries) {
			assertNull(e.getEvent());
		}
		
		//set null
		event.setEntries(null);
		assertEquals(new ArrayList<Entry>(), event.getEntries());
		for (Entry dj : diffEntries) {
			assertNull(dj.getEvent());
		}
	}
	
	@Test
	void testAddEntry() {
		//verify empty at first
		assertEquals(new ArrayList<Entry>(), event.getEntries());
		
		//add entry
		assertTrue(event.addEntry(entry));
		assertTrue(event.getEntries().contains(entry));
		assertEquals(event, entry.getEvent());
		
		//add another entry
		Entry entry2 = new Entry();
		assertTrue(event.addEntry(entry2));
		assertTrue(event.getEntries().contains(entry));
		assertTrue(event.getEntries().contains(entry2));
		assertEquals(event, entry.getEvent());
		assertEquals(event, entry2.getEvent());
		
		//try add null
		assertFalse(event.addEntry(null));
		assertFalse(event.getEntries().contains(null));
		assertEquals(2, event.getEntries().size());
		
		//try add duplicate
		assertFalse(event.addEntry(entry));
		assertTrue(event.getEntries().contains(entry));
		assertTrue(event.getEntries().contains(entry2));
		assertEquals(event, entry.getEvent());
		assertEquals(event, entry2.getEvent());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveEntryEntry() {
		//verify empty at first
		assertEquals(new ArrayList<Entry>(), event.getEntries());
		
		//set list
		List<Entry> jCopy = (List<Entry>)((ArrayList<Entry>)entries).clone();
		event.setEntries(jCopy);
		assertEquals(entries.size(), event.getEntries().size());
		
		//try to remove entry not in list
		assertFalse(event.removeEntry(new Entry()));
		assertEquals(entries.size(), event.getEntries().size());
		
		//try to remove null
		assertFalse(event.removeEntry(null));
		assertEquals(entries.size(), event.getEntries().size());
		
		//remove one by one
		for ( Entry j : entries ) {
			assertTrue(event.removeEntry(j));
		}
		assertEquals(0, event.getEntries().size());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveEntryInt() {
		//verify empty at first
		assertEquals(new ArrayList<Entry>(), event.getEntries());
		
		//set list
		List<Entry> jCopy = (List<Entry>)((ArrayList<Entry>)entries).clone();
		event.setEntries(jCopy);
		assertEquals(entries.size(), event.getEntries().size());
		
		//try to remove entry out of bounds
		try {
			assertNull(event.removeEntry(-1));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index -1 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(entries.size(), event.getEntries().size());
		
		try {
			assertNull(event.removeEntry(5));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index 5 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(entries.size(), event.getEntries().size());
		
		//remove one by one
		for ( int i = entries.size()-1; i >= 0; i--) {
			assertEquals(entries.get(i), event.removeEntry(i));
		}
		assertEquals(0, event.getEntries().size());
	}

	@SuppressWarnings("unchecked")
	@Test
	void testSetInviteCodes() {
		//verify empty at first
		assertEquals(new ArrayList<InviteCode>(), event.getInviteCodes());
		
		//set list
		List<InviteCode> icCopy = (List<InviteCode>)((ArrayList<InviteCode>)inviteCodes).clone();
		event.setInviteCodes(icCopy);
		assertIterableEquals(inviteCodes, event.getInviteCodes());
		for (InviteCode ic : inviteCodes) {
			assertEquals(event, ic.getEvent());
		}
		
		//set Same list:
		List<InviteCode> icCopy2 = (List<InviteCode>)((ArrayList<InviteCode>)inviteCodes).clone();
		event.setInviteCodes(icCopy2);
		assertIterableEquals(inviteCodes, event.getInviteCodes());
		for (InviteCode ic : inviteCodes) {
			assertEquals(event, ic.getEvent());
		}
		
		//prep different list
		List<InviteCode> diffInviteCodes = new ArrayList<InviteCode>();
		for (int i = 0; i < 3; i++) {
			diffInviteCodes.add(new InviteCode());
		}
		
		//set different list
		List<InviteCode> dicCopy = (List<InviteCode>)((ArrayList<InviteCode>)diffInviteCodes).clone();
		event.setInviteCodes(dicCopy);
		assertIterableEquals(diffInviteCodes, event.getInviteCodes());
		for (InviteCode dic : diffInviteCodes) {
			assertEquals(event, dic.getEvent());
		}
		for (InviteCode ic : inviteCodes) {
			assertNull(ic.getEvent());
		}
		
		//set null
		event.setInviteCodes(null);
		assertEquals(new ArrayList<InviteCode>(), event.getInviteCodes());
		for (InviteCode dic : diffInviteCodes) {
			assertNull(dic.getEvent());
		}
	}
	
	@Test
	void testAddInviteCode() {
		//verify empty at first
		assertEquals(new ArrayList<InviteCode>(), event.getInviteCodes());
		
		//add inviteCode
		assertTrue(event.addInviteCode(inviteCode));
		assertTrue(event.getInviteCodes().contains(inviteCode));
		assertEquals(event, inviteCode.getEvent());
		
		//add another inviteCode
		InviteCode inviteCode2 = new InviteCode();
		assertTrue(event.addInviteCode(inviteCode2));
		assertTrue(event.getInviteCodes().contains(inviteCode));
		assertTrue(event.getInviteCodes().contains(inviteCode2));
		assertEquals(event, inviteCode.getEvent());
		assertEquals(event, inviteCode2.getEvent());
		
		//try add null
		assertFalse(event.addInviteCode(null));
		assertFalse(event.getInviteCodes().contains(null));
		assertEquals(2, event.getInviteCodes().size());
		
		//try add duplicate
		assertFalse(event.addInviteCode(inviteCode));
		assertTrue(event.getInviteCodes().contains(inviteCode));
		assertTrue(event.getInviteCodes().contains(inviteCode2));
		assertEquals(event, inviteCode.getEvent());
		assertEquals(event, inviteCode2.getEvent());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveInviteCodeInviteCode() {
		//verify empty at first
		assertEquals(new ArrayList<InviteCode>(), event.getInviteCodes());
		
		//set list
		List<InviteCode> icCopy = (List<InviteCode>)((ArrayList<InviteCode>)inviteCodes).clone();
		event.setInviteCodes(icCopy);
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		//try to remove inviteCode not in list
		assertFalse(event.removeInviteCode(new InviteCode()));
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		//try to remove null
		assertFalse(event.removeInviteCode(null));
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		//remove one by one
		for ( InviteCode ic : inviteCodes ) {
			assertTrue(event.removeInviteCode(ic));
		}
		assertEquals(0, event.getInviteCodes().size());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	void testRemoveInviteCodeInt() {
		//verify empty at first
		assertEquals(new ArrayList<InviteCode>(), event.getInviteCodes());
		
		//set list
		List<InviteCode> icCopy = (List<InviteCode>)((ArrayList<InviteCode>)inviteCodes).clone();
		event.setInviteCodes(icCopy);
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		//try to remove inviteCode out of bounds
		try {
			assertNull(event.removeInviteCode(-1));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index -1 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		try {
			assertNull(event.removeInviteCode(5));
			fail( "Should have thrown Index out of Bounds exception." );
		} catch (Exception e) {
			String expectedMessage = "Index 5 out of bounds for length 5";
		    assertEquals(expectedMessage, e.getMessage());
		}
		assertEquals(inviteCodes.size(), event.getInviteCodes().size());
		
		//remove one by one
		for ( int i = inviteCodes.size()-1; i >= 0; i--) {
			assertEquals(inviteCodes.get(i), event.removeInviteCode(i));
		}
		assertEquals(0, event.getInviteCodes().size());
	}
	
	@Test
	void testSetInviteCounter() {
		//verify 0 at start
		assertEquals(0, event.getInviteCounter());
		
		//set value:
		int value = 5;
		event.setInviteCounter(value);
		assertEquals(value, event.getInviteCounter());
		
		//set value:
		value = -17;
		event.setInviteCounter(value);
		assertEquals(value, event.getInviteCounter());
		
		//set value:
		value = 0;
		event.setInviteCounter(value);
		assertEquals(value, event.getInviteCounter());
	}
	
	@Test
	void testIncInviteCounter() {
		//verify 0 at start
		assertEquals(0, event.getInviteCounter());
		
		assertEquals(1, event.incInviteCounter());
		assertEquals(1, event.getInviteCounter());
		
		assertEquals(2, event.incInviteCounter());
		assertEquals(2, event.getInviteCounter());
		
		assertEquals(3, event.incInviteCounter());
		assertEquals(3, event.getInviteCounter());
		
		event.setInviteCounter(100);
		
		assertEquals(101, event.incInviteCounter());
		assertEquals(101, event.getInviteCounter());
	}
	
}
