package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class InviteCodeTest {

	private InviteCode inviteCode;
	private Event event;
	
	@BeforeEach
	void setUp() throws Exception {
		inviteCode = new InviteCode();
		event = new Event();
		event.setId(123456789);
	}

	@Test
	void testInviteCode() {
		inviteCode = new InviteCode();
		assertNull(inviteCode.getCode());
		assertNull(inviteCode.getEvent());
	}

	@Test
	void testInviteCodeEvent() {
		//attempt with good args 
		inviteCode = new InviteCode(event);
		assertEquals(event, inviteCode.getEvent());
		assertTrue(event.getInviteCodes().contains(inviteCode));
		assertNotNull(inviteCode.getCode());
		
		//attempt with bad args
		inviteCode = new InviteCode(null);
		assertNull(inviteCode.getEvent());
		assertNull(inviteCode.getCode());
	}

	@Test
	void testSetEvent() {
		//verify null at first
		assertNull(inviteCode.getEvent());
		
		//set event
		inviteCode.setEvent(event);
		assertEquals(event, inviteCode.getEvent());
		assertTrue(event.getInviteCodes().contains(inviteCode));
		
		//set SAME event
		inviteCode.setEvent(event);
		assertEquals(event, inviteCode.getEvent());
		assertTrue(event.getInviteCodes().contains(inviteCode));
		
		//set different event
		Event event2 = new Event();
		inviteCode.setEvent(event2);
		assertEquals(event2, inviteCode.getEvent());
		assertTrue(event2.getInviteCodes().contains(inviteCode));
		assertFalse(event.getInviteCodes().contains(inviteCode));
		
		//set null
		inviteCode.setEvent(null);
		assertNull(inviteCode.getEvent());
		assertFalse(event2.getInviteCodes().contains(inviteCode));
	}

	@Test
	void testGenerateCode() {
		//try generating with no event
		assertFalse(inviteCode.generateCode());
		assertNull(inviteCode.getCode());
		
		//set event
		inviteCode.setEvent(event);
		
		assertTrue(inviteCode.generateCode());
		assertNotNull(inviteCode.getCode());

		assertTrue(inviteCode.generateCode());
		assertNotNull(inviteCode.getCode());

		assertTrue(inviteCode.generateCode());
		assertNotNull(inviteCode.getCode());
		
		assertEquals(3, event.getInviteCounter());
	}

}
