package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class ScoreTest {
	
	private Score score;
	private Score parentScore;
	private List<Score> subScores;
	
	private Category category;
	private Category parentCategory;
	private List<Category> subCategories;
	
	private Rubric rubric;
	private ScoredRubric scoredRubric;
	private Entry entry;

	@BeforeEach
	void setUp() throws Exception {
		score = new Score();
		parentScore = new Score();
		subScores = new ArrayList<Score>();
		for (int i = 0; i < 4; i++) {
			subScores.add(new Score());
		}
		
		category = new Category("Organization", 20);
		parentCategory = new Category("Presentation", 20);
		subCategories = new ArrayList<Category>();
		subCategories.add(new Category("Effectiveness", 5));
		subCategories.add(new Category("Clarity of Language", 5));
		subCategories.add(new Category("Grammar", 5));
		subCategories.add(new Category("Overall Visual Appeal and Clarity", 5));
		
		rubric = new Rubric();
		entry = new Entry("Why animals are cute");
		scoredRubric = new ScoredRubric(entry, rubric);
	}

	@Test
	void testScore() {
		score = new Score();
		assertNull(score.getTemplate());
		assertEquals(0, score.getPointsAwarded());
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
	}
	
	@Test
	void testScoreCategory() {
		//use normal parameters:
		score = new Score(category);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify other values:
		assertEquals(0, score.getPointsAwarded());
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());
		
		//pass null values
		score = new Score(null);
		
		//verify template is null:
		assertNull(score.getTemplate());
		
		//verify other values:
		assertEquals(0, score.getPointsAwarded());
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());
	}
	
	@Test
	void testScoreCategoryInt() {
		//use normal parameters:
		score = new Score(category, 15);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded:
		assertEquals(15, score.getPointsAwarded());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use null template parameters:
		score = new Score(null, 15);
		
		//verify template is null:
		assertNull(score.getTemplate());
		
		//verify points awarded not set b/c no template:
		assertEquals(0, score.getPointsAwarded());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use bad point value above:
		score = new Score(category, 25);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded not set:
		assertEquals(0, score.getPointsAwarded());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use bad point value below:
		score = new Score(category, -1);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded not set:
		assertEquals(0, score.getPointsAwarded());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use all bad params
		score = new Score(null, -1);
		
		//verify template relations:
		assertNull(score.getTemplate());
		
		//verify points awarded not set:
		assertEquals(0, score.getPointsAwarded());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());
	}
	
	@Test
	void testScoreCategoryIntScore() {
		//use normal parameters:
		score = new Score(category, 15, parentScore);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded:
		assertEquals(15, score.getPointsAwarded());

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify other values:
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use null template parameters:
		score = new Score(null, 15, parentScore);
		
		//verify template is null:
		assertNull(score.getTemplate());
		
		//verify points awarded not set b/c no template:
		assertEquals(0, score.getPointsAwarded());

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify other values:
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use bad point value above:
		score = new Score(category, 25, parentScore);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded not set:
		assertEquals(0, score.getPointsAwarded());

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify other values:
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());

		//use null parent:
		Score ps = null;
		score = new Score(category, 15, ps);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded set:
		assertEquals(15, score.getPointsAwarded());

		//verify parent relation null:
		assertNull(score.getParentScore());
		
		//verify other values:
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		assertNull(score.getScoredRubric());
	}
	
	@Test
	void testScoreCategoryIntScoredRubric() {
		//use normal parameters:
		score = new Score(category, 15, scoredRubric);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded:
		assertEquals(15, score.getPointsAwarded());
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		
		//use null template:
		score = new Score(null, 15, scoredRubric);
		
		//verify template relations:
		assertNull(score.getTemplate());
		
		//verify points awarded:
		assertEquals(0, score.getPointsAwarded());
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
		
		//use bad point value:
		score = new Score(category, 25, scoredRubric);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded:
		assertEquals(0, score.getPointsAwarded());
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());

		//use null scored rubric:
		ScoredRubric srNull = null;
		score = new Score(category, 15, srNull);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify points awarded:
		assertEquals(15, score.getPointsAwarded());
		
		//verify scored rubric relations:
		assertNull(score.getScoredRubric());
		
		//verify other values:
		assertNull(score.getParentScore());
		assertNotNull(score.getSubScores());
		assertEquals(0, score.getSubScores().size());
	}

	@Test
	@SuppressWarnings("unchecked")
	void testScoreCategoryScoreListOfScore() {
		//use normal parameters:
		setUpTree();
		parentScore.setSubScores(null);
		subScores.get(0).setPointsAwarded(4);
		subScores.get(1).setPointsAwarded(3);
		List<Score> ssCopy = (List<Score>)((ArrayList<Score>)subScores).clone();
		score = new Score(category, parentScore, subScores);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify sub score relations:
		assertIterableEquals(ssCopy, score.getSubScores());
		for (Score ssc : ssCopy) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(7, score.getPointsAwarded());
		assertEquals(7, parentScore.getPointsAwarded());
		
		//verify scored rubric:
		assertNull(score.getScoredRubric());
		
		//use null template:
		setUpTree();
		parentScore.setSubScores(null);
		subScores.get(0).setPointsAwarded(4);
		subScores.get(1).setPointsAwarded(3);
		List<Score> ssCopy2 = (List<Score>)((ArrayList<Score>)subScores).clone();
		score = new Score(null, parentScore, subScores);
		
		//verify template relations:
		assertNull(score.getTemplate());

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify sub score relations:
		assertIterableEquals(ssCopy2, score.getSubScores());
		for (Score ssc : ssCopy2) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(7, score.getPointsAwarded());
		assertEquals(7, parentScore.getPointsAwarded());
		
		//verify scored rubric:
		assertNull(score.getScoredRubric());

		//use null parent score:
		setUpTree();
		parentScore.setSubScores(null);
		subScores.get(0).setPointsAwarded(4);
		subScores.get(1).setPointsAwarded(3);
		List<Score> ssCopy3 = (List<Score>)((ArrayList<Score>)subScores).clone();
		Score psNull = null;
		score = new Score(category, psNull, subScores);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));

		//verify parent relation:
		assertNull(score.getParentScore());
		
		//verify sub score relations:
		assertIterableEquals(ssCopy3, score.getSubScores());
		for (Score ssc : ssCopy3) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(7, score.getPointsAwarded());
		assertEquals(7, parentScore.getPointsAwarded());
		
		//verify scored rubric:
		assertNull(score.getScoredRubric());

		//use null subscores:
		setUpTree();
		parentScore.setSubScores(null);
		score = new Score(category, parentScore, null);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));

		//verify parent relation:
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify sub score relations:
		assertEquals(new ArrayList<Score>(), score.getSubScores());
		
		//verify points awarded:
		assertEquals(0, score.getPointsAwarded());
		assertEquals(0, parentScore.getPointsAwarded());
		
		//verify scored rubric:
		assertNull(score.getScoredRubric());
	}
	
	@Test
	@SuppressWarnings("unchecked")
	void testScoreCategoryScoredRubricListOfScore() {
		//set up template trees:
		rubric.addCategory(category);
		category.setSubCategories(subCategories);
		
		//reference templates:
		scoredRubric.setTemplate(rubric);
		for (int i = 0; i < subScores.size(); i++) {
			subScores.get(i).setTemplate(subCategories.get(i));
		}
		
		//init some sub score point values:
		subScores.get(1).setPointsAwarded(4);
		subScores.get(2).setPointsAwarded(3);
		subScores.get(3).setPointsAwarded(5);
		
		//init score:
		List<Score> ssCopy = (List<Score>)((ArrayList<Score>)subScores).clone();
		score = new Score(category, scoredRubric, subScores);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify sub score relations:
		assertIterableEquals(ssCopy, score.getSubScores());
		for (Score ssc : ssCopy) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(12, score.getPointsAwarded());
		assertEquals(12, scoredRubric.getTotalScore());
		
		//verify parent score:
		assertNull(score.getParentScore());

		//set up template trees:
		rubric.addCategory(category);
		category.setSubCategories(subCategories);
		
		//reference templates:
		scoredRubric.setTemplate(rubric);
		for (int i = 0; i < subScores.size(); i++) {
			subScores.get(i).setTemplate(subCategories.get(i));
		}
		
		//init some sub score point values:
		subScores.get(1).setPointsAwarded(4);
		subScores.get(2).setPointsAwarded(3);
		subScores.get(3).setPointsAwarded(5);
		
		//init score with null template:
		scoredRubric.removeScoredCategory(0);
		List<Score> ssCopy2 = (List<Score>)((ArrayList<Score>)subScores).clone();
		score = new Score(null, scoredRubric, subScores);
		
		//verify template is null:
		assertNull(score.getTemplate());
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify sub score relations:
		assertIterableEquals(ssCopy2, score.getSubScores());
		for (Score ssc : ssCopy2) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(12, score.getPointsAwarded());
		assertEquals(12, scoredRubric.getTotalScore());
		
		//verify parent score:
		assertNull(score.getParentScore());
		
		//set up template trees:
		rubric.addCategory(category);
		category.setSubCategories(subCategories);
		
		//reference templates:
		for (int i = 0; i < subScores.size(); i++) {
			subScores.get(i).setTemplate(subCategories.get(i));
		}
		
		//init some sub score point values:
		subScores.get(1).setPointsAwarded(4);
		subScores.get(2).setPointsAwarded(3);
		subScores.get(3).setPointsAwarded(5);
		
		//init score with null scored rubric:
		List<Score> ssCopy3 = (List<Score>)((ArrayList<Score>)subScores).clone();
		ScoredRubric srNull = null;
		score = new Score(category, srNull, subScores);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify scored rubric is null:
		assertNull(score.getScoredRubric());
		
		//verify sub score relations:
		assertIterableEquals(ssCopy3, score.getSubScores());
		for (Score ssc : ssCopy3) {
			assertEquals(score, ssc.getParentScore());
		}
		
		//verify points awarded:
		assertEquals(12, score.getPointsAwarded());
		
		//verify parent score:
		assertNull(score.getParentScore());
		
		//set up template trees:
		rubric.addCategory(category);
		category.setSubCategories(subCategories);
		
		//reference templates:
		scoredRubric.setTemplate(rubric);
		
		//init score with null sub scores:
		scoredRubric.removeScoredCategory(0);
		score = new Score(category, scoredRubric, null);
		
		//verify template relations:
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//verify scored rubric relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//verify sub score is not null, but empty:
		assertIterableEquals(new ArrayList<Score>(), score.getSubScores());
		
		//verify points awarded:
		assertEquals(0, score.getPointsAwarded());
		assertEquals(0, scoredRubric.getTotalScore());
		
		//verify parent score:
		assertNull(score.getParentScore());
	}
	
	@Test
	void testSetTemplate() {
		//verify starting template is null:
		assertNull(score.getTemplate());
		
		//set template:
		score.setTemplate(category);
		
		//verify relation is set:
		assertEquals(category, score.getTemplate());
		
		//verify relation is reciprocated:
		assertTrue(category.getScores().contains(score));
		
		//set to SAME template:
		score.setTemplate(category);
		
		//verify relation is set:
		assertEquals(category, score.getTemplate());
		
		//verify relation is reciprocated:
		assertTrue(category.getScores().contains(score));
		
		//set new template:
		Category new_template = new Category("Other Cat", 15);
		score.setTemplate(new_template);
		
		//verify relation is set:
		assertEquals(new_template, score.getTemplate());
		
		//verify relation is reciprocated:
		assertTrue(new_template.getScores().contains(score));
		
		//verify old relation is not reciprocated:
		assertFalse(category.getScores().contains(score));
		
		//set null template:
		score.setTemplate(null);
		
		//verify relation is null:
		assertNull(score.getTemplate());
		
		//verify old relation is not reciprocated:
		assertFalse(new_template.getScores().contains(score));
	}
	
	@Test
	void testSetPointsAwarded() {
		//verify original value is 0:
		assertEquals(0, score.getPointsAwarded());
		
		//set new value without template:
		int newVal = 15;
		assertFalse(score.setPointsAwarded(newVal));
		assertEquals(0, score.getPointsAwarded());
		
		//set template and try setting new value:
		score.setTemplate(category);
		assertTrue(score.setPointsAwarded(newVal));
		assertEquals(newVal, score.getPointsAwarded());
		
		//try setting value below 0:
		assertFalse(score.setPointsAwarded(-1));
		assertEquals(newVal, score.getPointsAwarded());
		
		//try setting value over bounds:
		int overVal = category.getMaxScore() + 1;
		assertFalse(score.setPointsAwarded(overVal));
		assertEquals(newVal, score.getPointsAwarded());
		
		//set up score tree structure:
		setUpTree();
		
		//check starting score is 0:
		assertEquals(0, scoredRubric.getTotalScore());
		
		//try to change parent value:
		int newSubVal = 4;
		assertFalse(parentScore.setPointsAwarded(newSubVal));
		assertEquals(0, parentScore.getPointsAwarded());
		
		//change sub score value and verify parent values have changed:
		assertTrue(subScores.get(0).setPointsAwarded(newSubVal));
		assertEquals(newSubVal, subScores.get(0).getPointsAwarded());
		assertEquals(newSubVal, parentScore.getPointsAwarded());
		assertEquals(newSubVal, scoredRubric.getTotalScore());
		
		//change another sub score value and verify parent values have changed:
		int newSubVal2 = 3;
		int totalVal = newSubVal + newSubVal2;
		assertTrue(subScores.get(2).setPointsAwarded(newSubVal2));
		assertEquals(newSubVal2, subScores.get(2).getPointsAwarded());
		assertEquals(totalVal, parentScore.getPointsAwarded());
		assertEquals(totalVal, scoredRubric.getTotalScore());
	}
	
	@Test
	void testSetParentScore() {
		//verify starting parent is null:
		assertNull(score.getParentScore());
		
		//set parent:
		score.setParentScore(parentScore);
		
		//verify relation:
		assertEquals(parentScore, score.getParentScore());
		
		//verify relation is reciprocated:
		assertTrue(parentScore.getSubScores().contains(score));
		
		//set SAME parent:
		int pssSize = parentScore.getSubScores().size();
		score.setParentScore(parentScore);
		
		//verify relation:
		assertEquals(parentScore, score.getParentScore());
		
		//verify relation is reciprocated:
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify parent's sub scores didn't increase in size:
		assertEquals(pssSize, parentScore.getSubScores().size());
		
		//remove parent:
		score.setParentScore(null);
		
		//verify relation is gone:
		assertNull(score.getParentScore());
		
		//verify old relation is not reciprocated:
		assertFalse(parentScore.getSubScores().contains(score));
		
		//set parent to have other sub scores:
		@SuppressWarnings("unchecked")
		List<Score> ssCopy = (List<Score>)((ArrayList<Score>)subScores).clone();
		parentScore.setSubScores(subScores);
		
		//set parent:
		score.setParentScore(parentScore);
		
		//verify relation:
		assertEquals(parentScore, score.getParentScore());
		
		//verify relation is reciprocated:
		assertTrue(parentScore.getSubScores().contains(score));
		
		//verify parents old sub scores are still intact:
		for (Score ss : ssCopy) {
			assertTrue(parentScore.getSubScores().contains(ss));
		}
		
		//remove parent:
		score.setParentScore(null);
		
		//verify relation is gone:
		assertNull(score.getParentScore());
		
		//verify old relation is not reciprocated:
		assertFalse(parentScore.getSubScores().contains(score));
		
		//verify parents old sub scores are still intact:
		assertIterableEquals(ssCopy, parentScore.getSubScores());
	}
	
	@Test
	@SuppressWarnings("unchecked")
	void testSetSubScores() {
		//verify starting sub scores is empty:
		assertEquals(0, score.getSubScores().size());
		
		//set sub scores:
		List<Score> ssCopy = (List<Score>)((ArrayList<Score>)subScores).clone();
		score.setSubScores(subScores);
		
		//verify relation set:
		assertIterableEquals(ssCopy, score.getSubScores());
		
		//verify relations are reciprocated:
		for (Score ss : ssCopy) {
			assertEquals(score, ss.getParentScore());
		}
		
		//set SAME sub scores:
		List<Score> ssCopy2 = (List<Score>)((ArrayList<Score>)subScores).clone();
		score.setSubScores(ssCopy2);
		
		//verify relations are still set:
		assertIterableEquals(ssCopy2, score.getSubScores());
		
		//verify relations are still reciprocated:
		for (Score ss : ssCopy2) {
			assertEquals(score, ss.getParentScore());
		}
		
		//set sub scores to empty:
		score.setSubScores(null);
		
		//verify relations are gone:
		assertEquals(0, score.getSubScores().size());
		
		//verify relations are not reciprocated:
		for (Score ss : ssCopy2) {
			assertNull(ss.getParentScore());
		}
		
		//set sub scores to another parent and change:
		parentScore.setSubScores(subScores);
		score.setSubScores(ssCopy2);
		
		//verify relations are set:
		assertEquals(0, parentScore.getSubScores().size());
		assertIterableEquals(ssCopy2, score.getSubScores());
		
		//verify relations are reciprocated:
		for (Score ss : ssCopy2) {
			assertEquals(score, ss.getParentScore());
		}
	}
	
	@Test
	void testAddSubScore() {
		//add sub score to parent:
		parentScore.addSubScore(score);
		
		//verify relations:
		assertEquals(1, parentScore.getSubScores().size());
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//remove parent:
		score.setParentScore(null);
		
		//set up parent:
		parentScore.setSubScores(subScores);
		int ssSize = parentScore.getSubScores().size();
		
		//add another sub score:
		parentScore.addSubScore(score);
		
		//verify relations:
		assertEquals(ssSize+1, parentScore.getSubScores().size());
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
		
		//add duplicate:
		parentScore.addSubScore(score);
		
		//verify relations and that size has not increased:
		assertEquals(ssSize+1, parentScore.getSubScores().size());
		assertEquals(parentScore, score.getParentScore());
		assertTrue(parentScore.getSubScores().contains(score));
	}
	
	@Test
	void testRemoveSubScoreInt() {
		//give parent sub scores:
		parentScore.setSubScores(subScores);
		
		//verify sub scores were added:
		assertNotEquals(0, parentScore.getSubScores().size());
		
		//remove one by one:
		while ( parentScore.getSubScores().size() > 0 ) {
			Score scoreRemoved = parentScore.removeSubScore(0);
			
			//verify relation was removed:
			assertNull(scoreRemoved.getParentScore());
			assertFalse(parentScore.getSubScores().contains(scoreRemoved));
		}
		
		//verify empty:
		assertEquals(0, parentScore.getSubScores().size());
		
		//give parent sub scores:
		parentScore.setSubScores(subScores);
		
		//verify sub scores were added:
		assertNotEquals(0, parentScore.getSubScores().size());
		
		//remove one by one in different order:
		for (int i = parentScore.getSubScores().size()-1; i >= 0; i--) {
			Score scoreRemoved = parentScore.removeSubScore(i);
			
			//verify relation was removed:
			assertNull(scoreRemoved.getParentScore());
			assertFalse(parentScore.getSubScores().contains(scoreRemoved));
		}
		
		//verify empty:
		assertEquals(0, parentScore.getSubScores().size());
	}
	
	@Test
	void testRemoveSubScore() {
		//give parent sub scores:
		@SuppressWarnings("unchecked")
		List<Score> ssCopy = (List<Score>)((ArrayList<Score>)subScores).clone();
		parentScore.setSubScores(subScores);
		
		//verify sub scores were added:
		assertNotEquals(0, parentScore.getSubScores().size());
		
		//try removing unrelated sub score:
		assertFalse(parentScore.removeSubScore(score));
		
		//remove sub scores one by one:
		for (Score ss : ssCopy) {
			assertTrue(parentScore.removeSubScore(ss));
			assertNull(ss.getParentScore());
		}
		
		//verify empty set:
		assertEquals(0, parentScore.getSubScores().size());
	}
	
	void testSetScoredRubric() {
		//verify no scored rubric as start:
		assertNull(score.getScoredRubric());
		
		//set scored rubric:
		score.setScoredRubric(scoredRubric);
		
		//verify relations:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//set same scored rubric:
		score.setScoredRubric(scoredRubric);
		
		//verify relations haven't changed:
		assertEquals(scoredRubric, score.getScoredRubric());
		assertEquals(1, scoredRubric.getScoredCategories().size());
		assertTrue(scoredRubric.getScoredCategories().contains(score));
		
		//change scored rubric:
		ScoredRubric scoredRubric2 = new ScoredRubric();
		score.setScoredRubric(scoredRubric2);
		
		//verify relations have changed:
		assertEquals(scoredRubric2, score.getScoredRubric());
		assertFalse(scoredRubric.getScoredCategories().contains(score));
		assertTrue(scoredRubric2.getScoredCategories().contains(score));
		
		//remove scored rubric:
		score.setScoredRubric(null);
		
		//verify relations have been removed:
		assertNull(score.getScoredRubric());
		assertFalse(scoredRubric2.getScoredCategories().contains(score));
	}
	
	/**
	 * Sets up a tree structure using the rubric, 
	 * parentCategory, and subCateogry elements, plus 
	 * their scored counterparts. The structure does not 
	 * include category or score. 
	 */
	private void setUpTree() {
		//set up template trees:
		rubric.addCategory(parentCategory);
		parentCategory.setSubCategories(subCategories);
		
		//reference templates:
		scoredRubric.setTemplate(rubric);
		parentScore.setTemplate(parentCategory);
		for (int i = 0; i < subScores.size(); i++) {
			subScores.get(i).setTemplate(subCategories.get(i));
		}
		
		//set up scored tree:
		parentScore.setScoredRubric(scoredRubric);
		parentScore.setSubScores(subScores);
	}

}
