package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class ScoredRubricTest {

	private ScoredRubric scoredRubric;
	private Entry entry;
	private Rubric template;
	private Role judge;
	private ArrayList<Score> scoredCategories;

	@BeforeEach
	void setUp() throws Exception {
		scoredRubric = new ScoredRubric();
		entry = new Entry("Entry Test 1");
		template = new Rubric();
		template.setTotalMaxScore(100);
		judge = new Judge();
		scoredCategories = new ArrayList<Score>();

		for (int i = 0; i < 3; i++) {
			scoredCategories.add(new Score());
		}
	}

	@Test
	void testScoredRubric() {
		// new empty scoredRubric
		scoredRubric = new ScoredRubric();
		assertNull(scoredRubric.getEntry());
		assertNull(scoredRubric.getTemplate());
		assertNull(scoredRubric.getJudge());
		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());
	}

	@Test
	void testScoredRubricEntryRubric() {
		// test with every parameter:
		scoredRubric = new ScoredRubric(entry, template);
		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));
		assertEquals(template, scoredRubric.getTemplate());
		assertNull(scoredRubric.getJudge());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));
		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());

		// test with null parameters:
		scoredRubric = new ScoredRubric(null, null);
		assertNull(scoredRubric.getEntry());
		assertNull(scoredRubric.getTemplate());
		assertNull(scoredRubric.getJudge());
		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());
	}

	@Test
	void testScoredRubricEntryRoleRubric() {
		// test with every parameter:
		scoredRubric = new ScoredRubric(entry, template, judge);

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(judge, scoredRubric.getJudge());
		assertTrue(judge.getScoredRubrics().contains(scoredRubric));

		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());

		// test with null entry param
		scoredRubric = new ScoredRubric(null, template, judge);

		assertNull(scoredRubric.getEntry());
		assertFalse(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(judge, scoredRubric.getJudge());
		assertTrue(judge.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());

		// test with null template param
		scoredRubric = new ScoredRubric(entry, null, judge);

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertNull(scoredRubric.getTemplate());
		assertFalse(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(judge, scoredRubric.getJudge());
		assertTrue(judge.getScoredRubrics().contains(scoredRubric));

		assertEquals(0, scoredRubric.getTotalScore());
		assertEquals(new ArrayList<Score>(), scoredRubric.getScoredCategories());
		assertNull(scoredRubric.getJudgeNote());
	}

	@SuppressWarnings("unchecked")
	@Test
	void testScoredRubricEntryRubricListOfScoreString() {
		// test with every parameters
		ArrayList<Score> oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		scoredRubric = new ScoredRubric(entry, template, scoredCategories, "abc");

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());

		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		assertEquals("abc", scoredRubric.getJudgeNote());

		assertNull(scoredRubric.getJudge());

		// test with null entry param
		oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		scoredRubric = new ScoredRubric(null, template, scoredCategories, "abc");

		assertNull(scoredRubric.getEntry());
		assertFalse(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());

		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		assertEquals("abc", scoredRubric.getJudgeNote());

		assertNull(scoredRubric.getJudge());

		// test with null template param
		oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		scoredRubric = new ScoredRubric(entry, null, scoredCategories, "abc");

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertNull(scoredRubric.getTemplate());
		assertFalse(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());

		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		assertEquals("abc", scoredRubric.getJudgeNote());

		assertNull(scoredRubric.getJudge());

		// test with null scoredCategory param

		oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		scoredRubric = new ScoredRubric(entry, template, null, "abc");

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertNotEquals(oldScoredCategories, scoredRubric.getScoredCategories());
		assertEquals(0, scoredRubric.getScoredCategories().size());

		for (Score osc : oldScoredCategories) {
			assertNotEquals(scoredRubric, osc.getScoredRubric());
		}

		assertEquals("abc", scoredRubric.getJudgeNote());

		// test with null judge note param

		oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		scoredRubric = new ScoredRubric(entry, template, scoredCategories, null);

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());

		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		assertNull(scoredRubric.getJudgeNote());

		assertNull(scoredRubric.getJudge());
	}

	@Test
	void testSetEntry() {
		// new empty scoredRubric
		scoredRubric = new ScoredRubric();
		assertNull(scoredRubric.getEntry());

		// set new entry
		scoredRubric.setEntry(entry);

		assertEquals(entry, scoredRubric.getEntry());
		assertTrue(entry.getScoredRubrics().contains(scoredRubric));

		// set different entry
		Entry entry2 = new Entry("Entry Test 2");

		scoredRubric.setEntry(entry2);

		assertEquals(entry2, scoredRubric.getEntry());
		assertTrue(entry2.getScoredRubrics().contains(scoredRubric));
		assertFalse(entry.getScoredRubrics().contains(scoredRubric));

		// set entry to null
		scoredRubric.setEntry(null);
		assertNull(scoredRubric.getEntry());
		assertFalse(entry2.getScoredRubrics().contains(scoredRubric));
	}

	@Test
	void testSetTotalScore() {
		// verify zero score at start:
		assertEquals(0, scoredRubric.getTotalScore());

		// try setting with no template:
		scoredRubric.setTotalScore(15);
		assertEquals(0, scoredRubric.getTotalScore());

		// add template:
		scoredRubric.setTemplate(template);

		// try setting below zero:
		scoredRubric.setTotalScore(-1);
		assertEquals(0, scoredRubric.getTotalScore());

		// try setting below over bound:
		scoredRubric.setTotalScore(101);
		assertEquals(0, scoredRubric.getTotalScore());

		// set value:
		scoredRubric.setTotalScore(50);
		assertEquals(50, scoredRubric.getTotalScore());
	}

	@Test
	void testSetTemplate() {
		// new empty scoredRubric
		scoredRubric = new ScoredRubric();
		assertNull(scoredRubric.getTemplate());

		// set new template
		scoredRubric.setTemplate(template);

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));

		// set same template
		int size = template.getScoredRubrics().size();
		scoredRubric.setTemplate(template);

		assertEquals(template, scoredRubric.getTemplate());
		assertTrue(template.getScoredRubrics().contains(scoredRubric));
		assertEquals(size, template.getScoredRubrics().size());

		// set different template
		Rubric template2 = new Rubric();

		scoredRubric.setTemplate(template2);

		assertEquals(template2, scoredRubric.getTemplate());
		assertTrue(template2.getScoredRubrics().contains(scoredRubric));
		assertFalse(template.getScoredRubrics().contains(scoredRubric));

		// set template to null
		scoredRubric.setTemplate(null);
		assertNull(scoredRubric.getTemplate());
		assertFalse(template2.getScoredRubrics().contains(scoredRubric));
	}

	@Test
	void testSetJudge() {
		// verify null at start
		assertNull(scoredRubric.getJudge());

		// set new judge
		scoredRubric.setJudge(judge);
		assertEquals(judge, scoredRubric.getJudge());
		assertTrue(judge.getScoredRubrics().contains(scoredRubric));

		// set SAME judge
		int size = judge.getScoredRubrics().size();
		scoredRubric.setJudge(judge);
		assertEquals(judge, scoredRubric.getJudge());
		assertTrue(judge.getScoredRubrics().contains(scoredRubric));
		assertEquals(size, judge.getScoredRubrics().size());

		// set different judge
		Role diffJudge = new Chair();
		scoredRubric.setJudge(diffJudge);
		assertEquals(diffJudge, scoredRubric.getJudge());
		assertTrue(diffJudge.getScoredRubrics().contains(scoredRubric));
		assertFalse(judge.getScoredRubrics().contains(scoredRubric));
	}

	@Test
	void testSetScoredCategories() {
		// test scores size as empty score list
		assertEquals(0, scoredRubric.getScoredCategories().size());
		@SuppressWarnings("unchecked")
		ArrayList<Score> oldScoredCategories = (ArrayList<Score>) scoredCategories.clone();

		// set new score list
		scoredRubric.setScoredCategories(scoredCategories);

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());
		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		// set SAME score list
		scoredRubric.setScoredCategories(scoredCategories);

		assertEquals(oldScoredCategories, scoredRubric.getScoredCategories());
		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		// set different score list
		ArrayList<Score> scoredCategories2 = new ArrayList<Score>();
		for (int i = 0; i < 2; i++) {
			scoredCategories2.add(new Score());
		}

		scoredRubric.setScoredCategories(scoredCategories2);

		assertNotEquals(scoredCategories, scoredRubric.getScoredCategories());

		for (Score osc : scoredRubric.getScoredCategories()) {
			assertEquals(scoredRubric, osc.getScoredRubric());
		}

		for (Score osc : scoredCategories) {
			assertNotEquals(scoredRubric, osc.getScoredRubric());
		}

		// set score list to null
		scoredRubric.setScoredCategories(null);
		assertNotEquals(scoredCategories2, scoredRubric.getScoredCategories());
		assertEquals(0, scoredRubric.getScoredCategories().size());

		for (Score osc : scoredCategories2) {
			assertNotEquals(scoredRubric, osc.getScoredRubric());
		}
	}

	@Test
	void testSetJudgeNote() {
		// test initial judge note
		assertNull(scoredRubric.getJudgeNote());

		// set the judge note to empty
		String test_note1 = "";
		scoredRubric.setJudgeNote(test_note1);

		assertNull(scoredRubric.getJudgeNote());

		// set new judge note
		String test_note2 = "abc";
		scoredRubric.setJudgeNote(test_note2);
		assertEquals(test_note2, scoredRubric.getJudgeNote());

		// set judge note to null
		String test_note3 = null;
		scoredRubric.setJudgeNote(test_note3);
		assertNull(scoredRubric.getJudgeNote());
	}
}
